import React__default, { createElement, useState, useRef, useEffect, forwardRef, useCallback, useImperativeHandle } from 'react';
import { WidthProvider, Responsive } from 'react-grid-layout';
import { ProgressSpinner } from 'primereact/progressspinner';
import axios from 'axios';
import moment from 'moment';
import { v4 } from 'uuid';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import PivotGrid, { FieldPanel, FieldChooser, Scrolling } from 'devextreme-react/pivot-grid';
import PivotGridDataSource from 'devextreme/ui/pivot_grid/data_source';
import ReactFC from 'react-fusioncharts';
import FusionCharts from 'fusioncharts';
import Charts from 'fusioncharts/fusioncharts.charts';
import FusionTheme from 'fusioncharts/themes/fusioncharts.theme.fusion';
import useResizeObserver from 'use-resize-observer';
import Popup from 'reactjs-popup';
import Charts$1 from 'fusioncharts/fusioncharts.widgets';
import 'devextreme/dist/css/dx.light.css';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends.apply(this, arguments);
}

function _objectDestructuringEmpty(obj) {
  if (obj == null) throw new TypeError("Cannot destructure undefined");
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var Session = function Session() {};

var Tools = /*#__PURE__*/function () {
  function Tools() {}

  Tools.GetLogitudeURL = function GetLogitudeURL() {
    var logitude_url = location.href.replace('index.html', '');

    if (location.href.indexOf('localhost') > -1) {
      logitude_url = 'http://localhost:9996';
    } else {
      var urlArr = location.href.split("/index.html");
      var url = urlArr[0];
      url = url.replace(url.substring(url.lastIndexOf('/'), url.length), "");
      logitude_url = url;
    }

    return logitude_url;
  };

  Tools.Map = function Map(from, to) {
    if (!from) return;
    if (!to) to = {};
    var keys = Object.keys(from);

    for (var _i = 0, _keys = keys; _i < _keys.length; _i++) {
      var k = _keys[_i];

      if (from[k]) {
        to[k] = from[k];
      }
    }
  };

  return Tools;
}();

var ApiServices = /*#__PURE__*/function () {
  function ApiServices() {
    this._apiUrl = Tools.GetLogitudeURL();
  }

  var _proto = ApiServices.prototype;

  _proto.getByFilters = function getByFilters(api, filters) {
    var token = this.getTokenFromStorage();
    var apiURL = "" + this._apiUrl + api;
    apiURL = this.ConcatFiltersToURL(filters, apiURL);
    return axios.get(apiURL, {
      headers: {
        'Content-Type': 'application/json',
        'Token': token
      }
    });
  };

  _proto.get = function get(api) {
    var token = this.getTokenFromStorage();
    var apiURL = "" + this._apiUrl + api;
    return axios.get(apiURL, {
      headers: {
        'Content-Type': 'application/json',
        'Token': token
      }
    });
  };

  _proto.post = function post(api, date, requestSignal) {
    if (requestSignal === void 0) {
      requestSignal = undefined;
    }

    var token = this.getTokenFromStorage();
    var apiURL = "" + this._apiUrl + api;
    return axios.post(apiURL, date, {
      headers: {
        'Content-Type': 'application/json',
        'Token': token
      },
      signal: requestSignal
    });
  };

  _proto.getTokenFromStorage = function getTokenFromStorage() {
    return Session.Token || '';
  };

  _proto.ConcatFiltersToURL = function ConcatFiltersToURL(filters, urlparameters) {
    var mykeys = Object.keys(filters);
    var addtionalFiltersValues = null;

    for (var i in mykeys) {
      var propName = mykeys[i];
      var propValue = filters[propName];
      var ignoreFilter = propName.indexOf("Operator") > 0 && propValue == "Equals" || propName == "AdditionalFilters";

      if (urlparameters != "?") {
        urlparameters = urlparameters.concat('&');
      }

      if (!ignoreFilter) {
        propValue = propValue ? encodeURIComponent(propValue) : '';
        urlparameters = urlparameters.concat(propName.concat('=').concat(propValue));
      }

      if (propName == "AdditionalFilters" && propValue.length > 0) {
        addtionalFiltersValues = JSON.stringify(propValue);
      }
    }

    if (addtionalFiltersValues) {
      urlparameters = urlparameters.concat("&AdditionalFilters=").concat(addtionalFiltersValues);
    }

    return urlparameters;
  };

  return ApiServices;
}();

var DashboardAnalyticsService = /*#__PURE__*/function () {
  function DashboardAnalyticsService() {
    this.controller = 'DashboardAnalytics';
    this._ApiServices = new ApiServices();
  }

  var _proto = DashboardAnalyticsService.prototype;

  _proto.getData = function getData(widget, signal) {
    if (signal === void 0) {
      signal = undefined;
    }

    return this._ApiServices.post("/api/" + this.controller + "/PostGetDataAnalytic", widget, signal);
  };

  _proto.getTargetAnalyticsData = function getTargetAnalyticsData(widget, signal) {
    if (signal === void 0) {
      signal = undefined;
    }

    return this._ApiServices.post("/api/" + this.controller + "/PostGetTargetDataAnalytic", widget, signal);
  };

  _proto.getKpiData = function getKpiData(widget) {
    return this._ApiServices.post("/api/" + this.controller + "/PostGetKpiData", widget);
  };

  _proto.getTableData = function getTableData(widget) {
    return this._ApiServices.post("/api/" + this.controller + "/PostGetTableData", widget);
  };

  _proto.getGaugeData = function getGaugeData(widget, signal) {
    if (signal === void 0) {
      signal = undefined;
    }

    return this._ApiServices.post("/api/" + this.controller + "/PostGetGaugeData", widget, signal);
  };

  _proto.getObjectTableNameByWidgetEntityId = function getObjectTableNameByWidgetEntityId(entityId) {
    return this._ApiServices.get("/api/analyticsfactsmetadatas/getsingle?id=" + entityId);
  };

  _proto.getPivotData = function getPivotData(widget) {
    return this._ApiServices.post("/api/DashboardAnalytics/PostGetPivotData", widget);
  };

  return DashboardAnalyticsService;
}();

var _path;

function _extends$1() {
  _extends$1 = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends$1.apply(this, arguments);
}

function SvgRedWarning(props) {
  return /*#__PURE__*/createElement("svg", _extends$1({
    xmlns: "http://www.w3.org/2000/svg",
    width: 25,
    height: 25,
    viewBox: "0 0 150 134"
  }, props), _path || (_path = /*#__PURE__*/createElement("path", {
    fill: "#CF2528",
    d: "M70.5 1.1c-1.7.4-4.2 2.1-5.7 3.6C60.7 9.1 1 113.4.4 117.1c-.8 5.3 3.1 11.9 8.6 14.6l4.4 2.3h123.2l4.4-2.3c8.3-4.2 11-11.8 7-19.6-5.9-11.7-60.5-105-62.8-107.4C83.8 3.2 81 1.5 79.1 1c-4.1-1.1-4.5-1.1-8.6.1zm11.6 37.8c3.6 3.6 3.8 8.6 1 19.9-1.1 4.3-2.7 11.5-3.6 16.2-2.6 13.8-2.3 13.2-4.8 12.8-1.8-.2-2.3-1.2-3-5.3-.4-2.8-2.1-11.3-3.9-19-4-18.2-4-21 .1-24.7 4.4-4 10.1-3.9 14.2.1zM80 95.7c3.9 2.7 4.9 4.5 5 8.6 0 8.8-10.8 13.1-17.1 6.8-8.2-8.2 2.6-22 12.1-15.4z"
  })));
}

var DateGroupCodes = {
  Full_Date: 'Full Date',
  Month: 'Month',
  Quarter: 'Quarter',
  Year: 'Year',
  Month_Year: 'Month & Year',
  Quarter_Year: 'Quarter & Year'
};

var getDateFormat = function getDateFormat(label, _format) {
  if (!label) return label;

  var _date = moment(label);

  if (!_date || !_date.isValid()) return label;
  return _date.format(_format);
};

var getQuarterYearFormat = function getQuarterYearFormat(label) {
  if (!label) return label;

  if (label.includes("/")) {
    var result = label.split("/");
    var quarterVal = result && result[1] ? result[1] : "";
    var yearVal = result && result[0] ? result[0] : "";
    return quarterVal + " " + yearVal;
  }

  return label;
};

var getLabelFormatByDateGroupType = function getLabelFormatByDateGroupType(label, widget, isSecondGrouping) {
  if (isSecondGrouping === void 0) {
    isSecondGrouping = false;
  }

  if (!widget) return label;
  var groupFieldDataType = isSecondGrouping ? widget.SecondaryDateGroupCode : widget.DateGroupCode;
  if (!groupFieldDataType) return label;

  switch (groupFieldDataType) {
    case DateGroupCodes.Full_Date:
      return getDateFormat(label, "DD MMM YYYY");

    case DateGroupCodes.Month:
      return getDateFormat(label, "MMM");

    case DateGroupCodes.Quarter:
      return label;

    case DateGroupCodes.Year:
      return getDateFormat(label, "YYYY");

    case DateGroupCodes.Month_Year:
      return getDateFormat(label, "MMM YYYY");

    case DateGroupCodes.Quarter_Year:
      return getQuarterYearFormat(label);

    default:
      return label;
  }
};

var getTodayDates = function getTodayDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().format(format);
  var toDate = moment().format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getYesterdayDates = function getYesterdayDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().subtract(1, 'days').format(format);
  var toDate = moment().subtract(1, 'days').format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getTomorrowDates = function getTomorrowDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().add(1, 'days').format(format);
  var toDate = moment().add(1, 'days').format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getCurrentWeekDates = function getCurrentWeekDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().startOf('week').format(format);
  var toDate = moment().endOf('week').format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getCurrentMonthDates = function getCurrentMonthDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().startOf('month').format(format);
  var toDate = moment().endOf('month').format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getPreviousMonthDates = function getPreviousMonthDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().subtract(1, 'months').startOf('month').format(format);
  var toDate = moment().subtract(1, 'months').endOf('month').format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getCurrentMonthToDateDates = function getCurrentMonthToDateDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().startOf('month').format(format);
  var toDate = moment().format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getPreviousMonthToDateDates = function getPreviousMonthToDateDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().subtract(1, 'months').startOf('month').format(format);
  var toDate = moment().format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getCurrentQuarterDates = function getCurrentQuarterDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().startOf('quarter').format(format);
  var toDate = moment().endOf('quarter').format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getCurrentYearDates = function getCurrentYearDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().startOf('year').format(format);
  var toDate = moment().endOf('year').format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getPreviousYearDates = function getPreviousYearDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().subtract(1, 'years').startOf('year').format(format);
  var toDate = moment().subtract(1, 'years').endOf('year').format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getCurrentYearToDateDates = function getCurrentYearToDateDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().startOf('year').format(format);
  var toDate = moment().format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getPreviousYearToDateDates = function getPreviousYearToDateDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().subtract(1, 'years').startOf('year').format(format);
  var toDate = moment().format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getPreviousQuarterDates = function getPreviousQuarterDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().subtract(1, 'quarters').startOf('quarter').format(format);
  var toDate = moment().subtract(1, 'quarters').endOf('quarter').format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getCurrentQuarterToDateDates = function getCurrentQuarterToDateDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().startOf('quarter').format(format);
  var toDate = moment().format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getPreviousQuarterToDateDates = function getPreviousQuarterToDateDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().subtract(1, 'quarters').startOf('quarter').format(format);
  var toDate = moment().format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getLast7DaysDates = function getLast7DaysDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().subtract(6, 'days').format(format);
  var toDate = moment().format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getLast30DaysDates = function getLast30DaysDates(format) {
  if (format === void 0) {
    format = "YYYYMMDD000000";
  }

  var dateValues = [];
  var fromDate = moment().subtract(29, 'days').format(format);
  var toDate = moment().format(format);
  dateValues.push(fromDate);
  dateValues.push(toDate);
  return dateValues;
};

var getDateRangesByCode = function getDateRangesByCode(code) {
  var result = {
    fromDate: null,
    toDate: null
  };

  if (!code) {
    return {
      fromDate: getTodayDates()[0],
      toDate: getTodayDates()[1]
    };
  }

  switch (code) {
    case "Current_Year":
      result.fromDate = getCurrentYearDates()[0];
      result.toDate = getCurrentYearDates()[1];
      break;

    case "Previous_Year":
      result.fromDate = getPreviousYearDates()[0];
      result.toDate = getPreviousYearDates()[1];
      break;

    case "Current_Year_To_Date":
      result.fromDate = getCurrentYearToDateDates()[0];
      result.toDate = getCurrentYearToDateDates()[1];
      break;

    case "Previous_Year_To_Date":
      result.fromDate = getPreviousYearToDateDates()[0];
      result.toDate = getPreviousYearToDateDates()[1];
      break;

    case "Current_Quarter":
      result.fromDate = getCurrentQuarterDates()[0];
      result.toDate = getCurrentQuarterDates()[1];
      break;

    case "Previous_Quarter":
      result.fromDate = getPreviousQuarterDates()[0];
      result.toDate = getPreviousQuarterDates()[1];
      break;

    case "Current_Quarter_To_Date":
      result.fromDate = getCurrentQuarterToDateDates()[0];
      result.toDate = getCurrentQuarterToDateDates()[1];
      break;

    case "Previous_Quarter_To_Date":
      result.fromDate = getPreviousQuarterToDateDates()[0];
      result.toDate = getPreviousQuarterToDateDates()[1];
      break;

    case "Current_Month":
      result.fromDate = getCurrentMonthDates()[0];
      result.toDate = getCurrentMonthDates()[1];
      break;

    case "Previous_Month":
      result.fromDate = getPreviousMonthDates()[0];
      result.toDate = getPreviousMonthDates()[1];
      break;

    case "Current_Month_To_Date":
      result.fromDate = getCurrentMonthToDateDates()[0];
      result.toDate = getCurrentMonthToDateDates()[1];
      break;

    case "Previous_Month_To_Date":
      result.fromDate = getPreviousMonthToDateDates()[0];
      result.toDate = getPreviousMonthToDateDates()[1];
      break;

    case "Today":
      result.fromDate = getTodayDates()[0];
      result.toDate = getTodayDates()[1];
      break;

    case "Yesterday":
      result.fromDate = getYesterdayDates()[0];
      result.toDate = getYesterdayDates()[1];
      break;

    case "Tomorrow":
      result.fromDate = getTomorrowDates()[0];
      result.toDate = getTomorrowDates()[1];
      break;

    case "Last 7 Days":
      result.fromDate = getLast7DaysDates()[0];
      result.toDate = getLast7DaysDates()[1];
      break;

    case "Last 30 Days":
      result.fromDate = getLast30DaysDates()[0];
      result.toDate = getLast30DaysDates()[1];
      break;

    case "This Week":
      result.fromDate = getCurrentWeekDates()[0];
      result.toDate = getCurrentWeekDates()[1];
      break;
  }

  return result;
};

var isNullOrUndefined = function isNullOrUndefined(val) {
  return val == null || val == undefined;
};

var isNullOrUndefinedOrEmpty = function isNullOrUndefinedOrEmpty(val) {
  return val == null || val == undefined || val == "";
};

var getGlobalNumberFormatting = function getGlobalNumberFormatting() {
  var numberFormatCode = Session.NumberFormatCode;
  var numberSeparatorSettings = {
    thousandsSeparator: ",",
    decimalSeparator: "."
  };
  if (isNullOrUndefinedOrEmpty(numberFormatCode)) return numberSeparatorSettings;

  switch (numberFormatCode) {
    case "CD":
      numberSeparatorSettings.thousandsSeparator = ",";
      numberSeparatorSettings.decimalSeparator = ".";
      break;

    case "DC":
      numberSeparatorSettings.thousandsSeparator = ".";
      numberSeparatorSettings.decimalSeparator = ",";
      break;

    case "AD":
      numberSeparatorSettings.thousandsSeparator = "'";
      numberSeparatorSettings.decimalSeparator = ".";
      break;

    default:
      numberSeparatorSettings.thousandsSeparator = ",";
      numberSeparatorSettings.decimalSeparator = ".";
      break;
  }

  return numberSeparatorSettings;
};

var KpiChart = function KpiChart(props) {
  var _widget$current16;

  var _useState = useState(true),
      isLoading = _useState[0],
      setIsLoading = _useState[1];

  var _useState2 = useState(),
      error = _useState2[0],
      setError = _useState2[1];

  var widget = useRef();

  var _useState3 = useState(),
      data = _useState3[0],
      setData = _useState3[1];

  var _useState4 = useState(false),
      isKPICustomSizeNeeded = _useState4[0],
      setIsKPICustomSizeNeeded = _useState4[1];

  var _useState5 = useState(v4()),
      triggerId = _useState5[0],
      setTriggerId = _useState5[1];

  useEffect(function () {
    var _props$customChartPro;

    widget.current = props.customChartProps.widget;
    HandleKPICustomSize();
    RefreshData();
    (_props$customChartPro = props.customChartProps.dataBinding) === null || _props$customChartPro === void 0 ? void 0 : _props$customChartPro.widgetUpdated.subscribe(function (updatedWidget) {
      var _widget$current;

      if ((updatedWidget === null || updatedWidget === void 0 ? void 0 : updatedWidget.key) != ((_widget$current = widget.current) === null || _widget$current === void 0 ? void 0 : _widget$current.key)) return;
      widget.current = _extends({}, updatedWidget);
      setTriggerId(v4());
    });
  }, []);
  useEffect(function () {
    HandleKPICustomSize();
  }, [props.customChartProps]);

  var HandleKPICustomSize = function HandleKPICustomSize() {
    var _isPreviewModeActive = props.customChartProps && props.customChartProps.isPreviewModeActive;

    var _isKPICustomResizeNeeded = isKPICustomResizeNeeded();

    setIsKPICustomSizeNeeded(_isKPICustomResizeNeeded && !_isPreviewModeActive);
  };

  var calculateWidgetKPISettingsDates = function calculateWidgetKPISettingsDates() {
    var _updatedWidget = _extends({}, widget.current);

    if (_updatedWidget && _updatedWidget.KPIComparisonSettings && _updatedWidget.KPIComparisonSettings.length) {
      var kpiSettings = JSON.parse(_updatedWidget.KPIComparisonSettings);
      var isKPISettingsActivated = !isNullOrUndefinedOrEmpty(kpiSettings.PeriodFieldId);

      if (isKPISettingsActivated) {
        var PeriodDateFilterDates = getDateRangesByCode(kpiSettings.PeriodDateFilter.DateTimeValueCode);
        var ComparePeriodDateFilterDates = kpiSettings.ComparisonPeriodType == 'Manually' ? getDateRangesByCode(kpiSettings.ComparePeriodDateFilter.DateTimeValueCode) : getDateRangesByCode(null);
        kpiSettings.PeriodDateFilter.FieldValue = PeriodDateFilterDates.fromDate;
        kpiSettings.PeriodDateFilter.FieldValue2 = PeriodDateFilterDates.toDate;
        kpiSettings.ComparePeriodDateFilter.FieldValue = ComparePeriodDateFilterDates.fromDate;
        kpiSettings.ComparePeriodDateFilter.FieldValue2 = ComparePeriodDateFilterDates.toDate;
        _updatedWidget.KPIComparisonSettings = JSON.stringify(kpiSettings);
      }

      return _updatedWidget;
    }

    return _updatedWidget;
  };

  var RefreshData = function RefreshData() {
    if (Session.Tenant == 0) {
      var emptyData = {};
      emptyData.Value = 0;
      setData(emptyData);
      setIsLoading(false);
      return;
    }

    var dashboardAnalyticsService = new DashboardAnalyticsService();
    var postWidget = calculateWidgetKPISettingsDates();
    postWidget.onChange = undefined;
    dashboardAnalyticsService.getKpiData(postWidget).then(function (result) {
      setData(result.data);
      setIsLoading(false);
    }, function (error) {
      var _error$response, _error$response$data, _error$response2, _error$response2$data;

      console.log('error', error);
      if (error !== null && error !== void 0 && (_error$response = error.response) !== null && _error$response !== void 0 && (_error$response$data = _error$response.data) !== null && _error$response$data !== void 0 && _error$response$data.ErrorMessage) setError(error === null || error === void 0 ? void 0 : (_error$response2 = error.response) === null || _error$response2 === void 0 ? void 0 : (_error$response2$data = _error$response2.data) === null || _error$response2$data === void 0 ? void 0 : _error$response2$data.ErrorMessage);else setError(error.message);
      setIsLoading(false);
    });
  };

  var kpiClick = function kpiClick() {
    var _widget$current2, _widget$current3, _widget$current4, _widget$current4$Widg;

    var measureFieldId;
    var currentWidgetMeasure = widget === null || widget === void 0 ? void 0 : (_widget$current2 = widget.current) === null || _widget$current2 === void 0 ? void 0 : _widget$current2.WidgetMeasures[0];
    if (widget !== null && widget !== void 0 && (_widget$current3 = widget.current) !== null && _widget$current3 !== void 0 && _widget$current3.WidgetMeasures) measureFieldId = widget === null || widget === void 0 ? void 0 : (_widget$current4 = widget.current) === null || _widget$current4 === void 0 ? void 0 : (_widget$current4$Widg = _widget$current4.WidgetMeasures[0]) === null || _widget$current4$Widg === void 0 ? void 0 : _widget$current4$Widg.MeasureFieldId;
    props.customChartProps.onSelectDataPoint({
      MeasureFieldId: measureFieldId,
      Widget: widget.current,
      MeasureCode: currentWidgetMeasure.MeasureCode,
      Formula: currentWidgetMeasure.Formula,
      GroupById: data === null || data === void 0 ? void 0 : data.GroupById
    });
  };

  var isComparisonIndicatorChecked = function isComparisonIndicatorChecked() {
    var _widget$current5;

    var _kpiSettings = (_widget$current5 = widget.current) === null || _widget$current5 === void 0 ? void 0 : _widget$current5.KPIComparisonSettings;

    if (_kpiSettings) {
      var _kpiSettingsAsObj = JSON.parse(_kpiSettings);

      return !isNullOrUndefined(_kpiSettingsAsObj.ComparisonIndicator) ? _kpiSettingsAsObj.ComparisonIndicator : false;
    }

    return false;
  };

  function getRatioComponent() {
    if (!(data !== null && data !== void 0 && data.CompareWithPrevious) || !isComparisonIndicatorChecked()) return null;
    return React__default.createElement("div", {
      className: "dl-kpi-ratio-container"
    }, getArrowComponent(), React__default.createElement("div", {
      style: {
        fontSize: getKPIFormattingPropertyValue("PercentageFontSize", "px")
      },
      className: 'dl-kpi-ratio-text'
    }, data === null || data === void 0 ? void 0 : data.Ratio, " %"));
  }

  function getArrowComponent() {
    var _widget$current6, _widget$current7, _widget$current8, _widget$current9, _widget$current10, _widget$current11, _widget$current12, _widget$current13;

    if (!data) return null;
    if ((data === null || data === void 0 ? void 0 : data.Ratio) == 0 || data.Value == data.ComparisonValue || (data.Value == "0" || data.Value == 0) && (data.ComparisonValue == "0" || data.ComparisonValue == 0)) return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-up-green"
    }), React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-down-red",
      style: {
        marginTop: "2px"
      }
    }));
    if ((data === null || data === void 0 ? void 0 : data.Ratio) > 1 && (_widget$current6 = widget.current) !== null && _widget$current6 !== void 0 && _widget$current6.Increase && ((_widget$current7 = widget.current) === null || _widget$current7 === void 0 ? void 0 : _widget$current7.Increase) == "Positive") return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-up-green"
    }));
    if ((data === null || data === void 0 ? void 0 : data.Ratio) > 1 && (_widget$current8 = widget.current) !== null && _widget$current8 !== void 0 && _widget$current8.Increase && ((_widget$current9 = widget.current) === null || _widget$current9 === void 0 ? void 0 : _widget$current9.Increase) == "Negative") return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-up-red"
    }));
    if ((data === null || data === void 0 ? void 0 : data.Ratio) < 1 && (_widget$current10 = widget.current) !== null && _widget$current10 !== void 0 && _widget$current10.Increase && ((_widget$current11 = widget.current) === null || _widget$current11 === void 0 ? void 0 : _widget$current11.Increase) == "Positive") return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-down-red"
    }));
    if ((data === null || data === void 0 ? void 0 : data.Ratio) < 1 && (_widget$current12 = widget.current) !== null && _widget$current12 !== void 0 && _widget$current12.Increase && ((_widget$current13 = widget.current) === null || _widget$current13 === void 0 ? void 0 : _widget$current13.Increase) == "Negative") return React__default.createElement("div", {
      className: "dl-kpi-arrow-container"
    }, React__default.createElement("span", {
      className: "dl-kpi-arrow-pic dl-kpi-arrow-down-green"
    }));
    return null;
  }

  function getPreviousValueComponent() {
    if (isNullOrUndefined(data) || isNullOrUndefined(data === null || data === void 0 ? void 0 : data.ComparisonValue) || isNullOrUndefined(data === null || data === void 0 ? void 0 : data.ComparisonLabel)) return "";
    var formatedValue = data ? data.ComparisonLabel : "";
    var comparisonValue = data ? data === null || data === void 0 ? void 0 : data.ComparisonValue : "";
    return React__default.createElement("div", {
      title: formatedValue,
      className: "dl-kpi-prev-container"
    }, React__default.createElement("div", {
      style: {
        color: getKPIFormattingPropertyValue("ComparisonValueFontColor"),
        fontSize: getKPIFormattingPropertyValue("ComparisonValueFontSize", "px")
      },
      className: "dl-kpi-prev-text"
    }, "" + comparisonValue + (isPercentageMeasureField() ? "%" : "")), (data === null || data === void 0 ? void 0 : data.ComparisonAbbreviationSymbol) && React__default.createElement("div", {
      style: {
        color: getKPIFormattingPropertyValue("ComparisonValueFontColor"),
        fontSize: getKPIFormattingPropertyValue("ComparisonValueFontSize", "px")
      },
      className: "dl-kpi-prev-abbreviationSymbol"
    }, data === null || data === void 0 ? void 0 : data.ComparisonAbbreviationSymbol), widget.current && widget.current.ShowMeasureUnit && (data === null || data === void 0 ? void 0 : data.Unit) && React__default.createElement("div", {
      style: {
        color: getKPIFormattingPropertyValue("ComparisonValueFontColor"),
        fontSize: getKPIFormattingPropertyValue("ComparisonValueFontSize", "px")
      },
      className: "comparison-value-dl-kpi-unit"
    }, data === null || data === void 0 ? void 0 : data.Unit));
  }

  var isPercentageMeasureField = function isPercentageMeasureField() {
    return (widget === null || widget === void 0 ? void 0 : widget.current) && (widget === null || widget === void 0 ? void 0 : widget.current.WidgetMeasures) && (widget === null || widget === void 0 ? void 0 : widget.current.WidgetMeasures[0]) && (widget === null || widget === void 0 ? void 0 : widget.current.WidgetMeasures[0].MeasureFieldDataType) && (widget === null || widget === void 0 ? void 0 : widget.current.WidgetMeasures[0].MeasureFieldDataType) == 'Percentage';
  };

  function getValueData() {
    if ((data === null || data === void 0 ? void 0 : data.Value) == null) return React__default.createElement("div", {
      className: "dl-kpi-no-data"
    }, "No data to display.");
    return React__default.createElement("div", {
      style: {
        color: getKPIFormattingPropertyValue("MainValueFontColor"),
        fontSize: getKPIFormattingPropertyValue("MainValueFontSize", "px")
      },
      className: "dl-kpi-value dl-kpi-textTrimming"
    }, "" + (data === null || data === void 0 ? void 0 : data.Value) + (isPercentageMeasureField() ? "%" : ""));
  }

  function getValueComponent() {
    return React__default.createElement("div", {
      className: "dl-kpi-sub-container"
    }, getValueData(), (data === null || data === void 0 ? void 0 : data.AbbreviationSymbol) && React__default.createElement("div", {
      style: {
        color: getKPIFormattingPropertyValue("MainValueFontColor"),
        fontSize: getKPIFormattingPropertyValue("MainValueFontSize", "px")
      },
      className: "dl-kpi-abbreviationSymbol"
    }, data === null || data === void 0 ? void 0 : data.AbbreviationSymbol), widget.current && widget.current.ShowMeasureUnit && (data === null || data === void 0 ? void 0 : data.Unit) && React__default.createElement("div", {
      style: {
        color: getKPIFormattingPropertyValue("MainValueFontColor"),
        fontSize: getKPIFormattingPropertyValue("MainValueFontSize", "px")
      },
      className: "dl-kpi-unit"
    }, data === null || data === void 0 ? void 0 : data.Unit), getRatioComponent());
  }

  var getPeriodLabelByPeriodCode = function getPeriodLabelByPeriodCode(code) {
    if (!isNullOrUndefinedOrEmpty(code)) {
      return code.replaceAll("_", " ");
    }

    return "";
  };

  var getLabelFromKPISettings = function getLabelFromKPISettings() {
    if (widget.current && widget.current.KPIComparisonSettings && widget.current.KPIComparisonSettings.length) {
      var kpiSettings = JSON.parse(widget.current.KPIComparisonSettings);
      var PeriodDateFilter = kpiSettings ? kpiSettings.PeriodDateFilter : null;
      var isKPISettingsActivated = !isNullOrUndefinedOrEmpty(kpiSettings.PeriodFieldId);
      return isKPISettingsActivated && PeriodDateFilter && PeriodDateFilter.DateTimeValueCode ? getPeriodLabelByPeriodCode(PeriodDateFilter.DateTimeValueCode) : "";
    }

    return "";
  };

  var getKPIFormattingData = function getKPIFormattingData() {
    var _widget$current14, _widget$current15;

    return isNullOrUndefinedOrEmpty((_widget$current14 = widget.current) === null || _widget$current14 === void 0 ? void 0 : _widget$current14.KPIFormatSettings) ? {} : JSON.parse((_widget$current15 = widget.current) === null || _widget$current15 === void 0 ? void 0 : _widget$current15.KPIFormatSettings);
  };

  var getKPIFormattingPropertyValue = function getKPIFormattingPropertyValue(propertyName, unit) {
    if (unit === void 0) {
      unit = "";
    }

    var kpiFormattingData = getKPIFormattingData();
    return kpiFormattingData && kpiFormattingData[propertyName] ? kpiFormattingData[propertyName] + unit : undefined;
  };

  function getLabel() {
    var formatedValue = data ? data.Label : "";
    var label = getLabelFromKPISettings();
    return React__default.createElement("div", {
      className: "dl-kpi-label-container"
    }, React__default.createElement("span", {
      style: {
        color: getKPIFormattingPropertyValue("MainPeriodFontColor"),
        fontSize: getKPIFormattingPropertyValue("MainPeriodFontSize", "px")
      },
      title: formatedValue,
      className: "dl-kpi-label-span dl-kpi-textTrimming"
    }, label));
  }

  var getIsKPIHasSmallestHeight = function getIsKPIHasSmallestHeight() {
    var _props$customChartPro2, _props$customChartPro3;

    return props.customChartProps.widgetRef && ((_props$customChartPro2 = props.customChartProps.widgetRef) === null || _props$customChartPro2 === void 0 ? void 0 : _props$customChartPro2.Layout) && ((_props$customChartPro3 = props.customChartProps.widgetRef.Layout) === null || _props$customChartPro3 === void 0 ? void 0 : _props$customChartPro3.h) == 2;
  };

  var isKPICustomResizeNeeded = function isKPICustomResizeNeeded() {
    if (!widget || !widget.current) return;
    var isKPIHasSmallestHeight = getIsKPIHasSmallestHeight();
    return isKPIHasSmallestHeight;
  };

  return isLoading || !widget.current ? React__default.createElement("div", {
    className: 'dl-full-hight dl-flex-content-center spinner-custome'
  }, React__default.createElement(ProgressSpinner, {
    style: {
      width: '40px',
      height: '40px'
    },
    strokeWidth: "4",
    animationDuration: "2s"
  })) : !error && widget.current ? React__default.createElement("div", {
    key: triggerId,
    onClick: kpiClick,
    className: ((_widget$current16 = widget.current) === null || _widget$current16 === void 0 ? void 0 : _widget$current16.Alighnment) == "Left" ? "dl-kpi-container-click db-justify-content-flex-start" : "dl-kpi-container-click"
  }, React__default.createElement("div", {
    className: isKPICustomSizeNeeded ? "dl-kpi-container kpi-custom-size" : "dl-kpi-container"
  }, getLabel(), getValueComponent(), getPreviousValueComponent())) : React__default.createElement("div", {
    className: "dl-flex-content-center dl-full-hight",
    style: {
      width: '100%'
    }
  }, React__default.createElement("div", {
    className: "dl-tooltip"
  }, React__default.createElement(SvgRedWarning, {
    className: "red-warning-icon"
  }), React__default.createElement("span", {
    className: "dl-tooltiptext"
  }, error)));
};

var TableChart = function TableChart(props) {
  var _useState = useState(true),
      isLoading = _useState[0],
      setIsLoading = _useState[1];

  var _useState2 = useState(false),
      isTableLoading = _useState2[0],
      setIsTableLoading = _useState2[1];

  var _useState3 = useState(),
      error = _useState3[0],
      setError = _useState3[1];

  var _useState4 = useState(),
      widget = _useState4[0],
      setWidget = _useState4[1];

  var _useState5 = useState([]),
      tableData = _useState5[0],
      setTableData = _useState5[1];

  var _useState6 = useState([]),
      tableColumns = _useState6[0],
      setTableColumns = _useState6[1];

  var _useState7 = useState(null),
      sortField = _useState7[0],
      setSortField = _useState7[1];

  var _useState8 = useState(null),
      sortOrder = _useState8[0],
      setSortOrder = _useState8[1];

  var isDefaultSortOptionsUsed = useRef(false);
  var widgetRefData = useRef(null);

  var _useState9 = useState(null),
      objectTableName = _useState9[0],
      setObjectTableName = _useState9[1];

  useEffect(function () {
    var _props$customChartPro;

    if (props.customChartProps && props.customChartProps.widget) {
      setWidget(props.customChartProps.widget);
      widgetRefData.current = props.customChartProps.widget;
      getObjectTableName(props.customChartProps.widget);
    }

    (_props$customChartPro = props.customChartProps.dataBinding) === null || _props$customChartPro === void 0 ? void 0 : _props$customChartPro.widgetUpdated.subscribe(function (updatedWidget) {
      var _widgetRefData$curren;

      if ((updatedWidget === null || updatedWidget === void 0 ? void 0 : updatedWidget.key) != ((_widgetRefData$curren = widgetRefData.current) === null || _widgetRefData$curren === void 0 ? void 0 : _widgetRefData$curren.key)) return;
      setWidget(_extends({}, updatedWidget));
    });
  }, []);

  var getObjectTableName = function getObjectTableName(widget) {
    var dashboardAnalyticsService = new DashboardAnalyticsService();
    dashboardAnalyticsService.getObjectTableNameByWidgetEntityId(widget === null || widget === void 0 ? void 0 : widget.EntityId).then(function (res) {
      if (res && res.data && res.data.ObjectTableName) {
        setObjectTableName(res.data.ObjectTableName);
        getTableData(widget);
      } else {
        setError("Erorr: No object table name match this widget entity id");
        setIsLoading(false);
      }
    })["catch"](function () {
      setError("Erorr: No object table name match this widget entity id");
      setIsLoading(false);
    });
  };

  var getTableData = function getTableData(widget) {
    if (Session.Tenant == 0) {
      setTableData([]);
      setIsLoading(false);
      return;
    }

    var dashboardAnalyticsService = new DashboardAnalyticsService();
    widget.onChange = undefined;
    widget.PageIndex = 0;
    widget.PageSize = 100;
    dashboardAnalyticsService.getTableData(widget).then(function (res) {
      if (res && res.data) {
        fillTableData(res.data);
      }

      setIsLoading(false);
    })["catch"](function (error) {
      var _error$response, _error$response$data, _error$response2, _error$response2$data;

      if (error !== null && error !== void 0 && (_error$response = error.response) !== null && _error$response !== void 0 && (_error$response$data = _error$response.data) !== null && _error$response$data !== void 0 && _error$response$data.ErrorMessage) setError(error === null || error === void 0 ? void 0 : (_error$response2 = error.response) === null || _error$response2 === void 0 ? void 0 : (_error$response2$data = _error$response2.data) === null || _error$response2$data === void 0 ? void 0 : _error$response2$data.ErrorMessage);else setError(error.message);
      setIsLoading(false);
    });
  };

  var getBooleanStringValue = function getBooleanStringValue(value) {
    if (isNullOrUndefined(value) || isNullOrUndefined(value.Label)) return "";

    var _value = value.Label + "";

    return {
      Label: _value.charAt(0).toUpperCase() + _value.slice(1),
      Value: _value.charAt(0).toUpperCase() + _value.slice(1)
    };
  };

  var getDateTimeValue = function getDateTimeValue(value) {
    if (!value) return value;

    var _date = moment(value);

    if (!_date || !_date.isValid()) return value;
    return _date.format("DD MMM YYYY");
  };

  var fillTableData = function fillTableData(tableData) {
    var tableRecords = [];

    if (tableData.Fields && tableData.Fields.length) {
      setTableColumns([].concat(tableData.Fields));
      setDefaultSortOptions(tableData.Fields);
    }

    if (tableData.Result && tableData.Result.length) {
      tableData.Result.forEach(function (item) {
        var _tableRow = _extends({}, item);

        tableData.Fields.forEach(function (field) {
          var key = field.FieldCode;
          var valueType = field.DataTypeCode;

          var _value = valueType && valueType == "Boolean" ? getBooleanStringValue(item[key]) : item[key];

          _tableRow[key] = isNullOrUndefined(_value) || isNullOrUndefined(_value.Value) && isNullOrUndefined(_value.Label) ? "" : _value;
        });
        tableRecords.push(_tableRow);
      });
    }

    setTableData([].concat(tableRecords));
  };

  var setDefaultSortOptions = function setDefaultSortOptions(tableCols) {
    if (tableCols === void 0) {
      tableCols = [];
    }

    if (isDefaultSortOptionsUsed && isDefaultSortOptionsUsed.current) return;
    isDefaultSortOptionsUsed.current = true;

    var _widget = props.customChartProps ? props.customChartProps.widget : null;

    if (!_widget) return;

    var _sortFieldCode = getSortFieldCodeBySortFieldId(_widget.DefaultSortBy, tableCols);

    var _sortDirection = _widget.DefaultSortDirection ? _widget.DefaultSortDirection == 'asc' ? 1 : _widget.DefaultSortDirection == 'desc' ? -1 : 0 : null;

    setSortField(_sortFieldCode);
    setSortOrder(_sortDirection);
  };

  var getSortFieldCodeBySortFieldId = function getSortFieldCodeBySortFieldId(sortFieldId, tableCols) {
    if (tableCols === void 0) {
      tableCols = [];
    }

    if (!sortFieldId) return null;
    var field = tableCols.find(function (col) {
      return col.Id == sortFieldId;
    });
    return field ? field.FieldCode : null;
  };

  var getSortFieldIdBySortFieldCode = function getSortFieldIdBySortFieldCode(sortFieldCode) {
    if (!sortFieldCode) return null;
    var field = tableColumns.find(function (col) {
      return col.FieldCode == sortFieldCode;
    });
    return field ? field.Id : null;
  };

  var sortTableData = function sortTableData(sortedField, isCalculatedColumn) {
    if (!canSort()) return;
    setSortField(sortedField);

    var _sortOrder = sortOrder == 1 ? -1 : sortOrder == -1 ? 1 : 1;

    setSortOrder(_sortOrder);
    var _widget = props.customChartProps.widget;
    var dashboardAnalyticsService = new DashboardAnalyticsService();
    _widget.onChange = undefined;
    _widget.PageIndex = 0;
    _widget.PageSize = 100;
    _widget.ScreenSortBy = getSortFieldIdBySortFieldCode(sortedField);
    _widget.ScreenSortDirection = _sortOrder == 1 ? 'asc' : _sortOrder == -1 ? 'desc' : null;
    _widget.ScreenSortByType = isNullOrUndefinedOrEmpty(isCalculatedColumn) ? "Field" : isCalculatedColumn ? "CalculatedColumn" : "Field";
    setIsTableLoading(true);
    dashboardAnalyticsService.getTableData(_widget).then(function (res) {
      if (res && res.data) {
        fillTableData(res.data);
      }

      setIsTableLoading(false);
    })["catch"](function (error) {
      var _error$response3, _error$response3$data, _error$response4, _error$response4$data;

      if (error !== null && error !== void 0 && (_error$response3 = error.response) !== null && _error$response3 !== void 0 && (_error$response3$data = _error$response3.data) !== null && _error$response3$data !== void 0 && _error$response3$data.ErrorMessage) setError(error === null || error === void 0 ? void 0 : (_error$response4 = error.response) === null || _error$response4 === void 0 ? void 0 : (_error$response4$data = _error$response4.data) === null || _error$response4$data === void 0 ? void 0 : _error$response4$data.ErrorMessage);else setError(error.message);
      setIsTableLoading(false);
    });
  };

  var canSort = function canSort() {
    if (props.customChartProps) {
      return !props.customChartProps.isInEditMode && !props.customChartProps.isPreviewModeActive;
    }

    return false;
  };

  var getTableFormattingData = function getTableFormattingData() {
    return widget && isNullOrUndefinedOrEmpty(widget.TableFormatSettings) ? {} : JSON.parse(widget === null || widget === void 0 ? void 0 : widget.TableFormatSettings);
  };

  var getTableFormattingPropertyValue = function getTableFormattingPropertyValue(propertyName, unit) {
    if (unit === void 0) {
      unit = "";
    }

    var tableFormattingData = getTableFormattingData();
    return tableFormattingData && tableFormattingData[propertyName] ? tableFormattingData[propertyName] + unit : undefined;
  };

  var getTableConditionalRulesData = function getTableConditionalRulesData() {
    return widget && isNullOrUndefinedOrEmpty(widget.TableConditionalRules) ? [] : JSON.parse(widget === null || widget === void 0 ? void 0 : widget.TableConditionalRules);
  };

  var isValueEmpty = function isValueEmpty(val) {
    return val === null || val === undefined || val === "";
  };

  var IsValidRule = function IsValidRule(rule, value) {
    var operator = rule && rule.RuleOperator ? rule.RuleOperator.Code : null;
    if (!operator) return null;
    var ruleValue1 = rule.RuleValue1;
    var ruleValue2 = rule.RuleValue2;

    switch (operator) {
      case "Equal":
        return ruleValue1 == value;

      case "NotEqual":
        return ruleValue1 != value;

      case "Between":
        return !isValueEmpty(ruleValue1) && !isValueEmpty(ruleValue2) && ruleValue1 <= value && value <= ruleValue2;

      case "GreaterThan":
        return !isValueEmpty(ruleValue1) && value > ruleValue1;

      case "GreaterThanOrEqual":
        return !isValueEmpty(ruleValue1) && value >= ruleValue1;

      case "LessThan":
        return !isValueEmpty(ruleValue1) && value < ruleValue1;

      case "LessThanOrEqual":
        return !isValueEmpty(ruleValue1) && value <= ruleValue1;

      case "IsEmpty":
        return isValueEmpty(value);

      case "IsNotEmpty":
        return !isValueEmpty(value);

      case "Contains":
        return !isValueEmpty(value) && !isValueEmpty(ruleValue1) && value.includes(ruleValue1);

      case "NotContains":
        return !isValueEmpty(value) && !isValueEmpty(ruleValue1) && !value.includes(ruleValue1);

      default:
        return null;
    }
  };

  var getCellPropertyByRule = function getCellPropertyByRule(column, cellValue, propertyName, defaultValue) {
    if (defaultValue === void 0) {
      defaultValue = null;
    }

    var tableFormattingData = getTableConditionalRulesData();
    var columnType = column.IsCalculatedColumn ? "CalculatedColumn" : "Field";
    var columnData = tableFormattingData.find(function (el) {
      return el.FieldId == column.Id && el.FieldType == columnType;
    });

    if (!columnData || !columnData.FieldRules || columnData.FieldRules.length == 0) {
      return defaultValue ? getTableFormattingPropertyValue(defaultValue) : undefined;
    }
    var firstValidRule = columnData.FieldRules.find(function (rule) {
      return IsValidRule(rule, cellValue);
    });

    if (!firstValidRule) {
      return defaultValue ? getTableFormattingPropertyValue(defaultValue) : undefined;
    }

    return !isNullOrUndefinedOrEmpty(firstValidRule[propertyName]) ? firstValidRule[propertyName] : defaultValue ? getTableFormattingPropertyValue(defaultValue) : undefined;
  };

  var getCellFloatAlignment = function getCellFloatAlignment(column) {
    if (column.DataTypeCode == 'Decimal' || column.DataTypeCode == 'Percentage' || column.DataTypeCode == 'Integer' || column.DataTypeCode == 'Double' || column.DataTypeCode == 'SigDouble') {
      return "right";
    }

    return "unset";
  };

  var isClickableCell = function isClickableCell(fieldCode) {
    return fieldCode == "ShipmentNumber" && objectTableName == "Shipment" || fieldCode == "InvoiceNumber" && objectTableName == "APInvoice" || fieldCode == "InvoiceNumber" && objectTableName == "ARInvoice" || fieldCode == "ContainerNumber" && objectTableName == "Container" || fieldCode == "QuoteNumber" && objectTableName == "Quote" || fieldCode == "Subject" && objectTableName == "Opportunity" || fieldCode == "EnglishName" && objectTableName == "Customer";
  };

  var tableCellClicked = function tableCellClicked(data, fieldCode) {
    if (!data || !data.Id || !data.Id.Label || !isClickableCell(fieldCode) || props.customChartProps.isInEditMode) return;
    props.customChartProps.onSelectDataPoint({
      ObjectTableName: objectTableName,
      TableRowEntityId: data.Id.Label
    });
  };

  var cellTemplateBody = function cellTemplateBody(data, column) {
    var _data$column$FieldCod, _data$column$FieldCod2, _data$column$FieldCod3, _data$column$FieldCod4, _data$column$FieldCod5, _data$column$FieldCod6, _data$column$FieldCod7;

    if (!column || !column.FieldCode) return "";
    return column.DataTypeCode == "DateTime" || column.DataTypeCode == "Date" ? React__default.createElement("span", {
      title: getDateTimeValue((_data$column$FieldCod = data[column.FieldCode]) === null || _data$column$FieldCod === void 0 ? void 0 : _data$column$FieldCod.Label),
      className: "table-cell",
      style: {
        color: getTableFormattingPropertyValue("TableCellFontColor"),
        fontSize: getTableFormattingPropertyValue("TableCellFontSize", "px")
      }
    }, getDateTimeValue((_data$column$FieldCod2 = data[column.FieldCode]) === null || _data$column$FieldCod2 === void 0 ? void 0 : _data$column$FieldCod2.Label)) : React__default.createElement("span", {
      onClick: function onClick() {
        return tableCellClicked(data, column.FieldCode);
      },
      title: (_data$column$FieldCod3 = data[column.FieldCode]) === null || _data$column$FieldCod3 === void 0 ? void 0 : _data$column$FieldCod3.Label,
      className: "table-cell " + (isClickableCell(column.FieldCode) ? 'table-link-cell' : ''),
      style: {
        color: getTableFormattingPropertyValue("TableCellFontColor"),
        backgroundColor: getCellPropertyByRule(column, (_data$column$FieldCod4 = data[column.FieldCode]) === null || _data$column$FieldCod4 === void 0 ? void 0 : _data$column$FieldCod4.Value, 'RuleBackgroundColor'),
        fontSize: getTableFormattingPropertyValue("TableCellFontSize", "px"),
        "float": getCellFloatAlignment(column)
      }
    }, column.DataTypeCode == "Percentage" ? (_data$column$FieldCod5 = data[column.FieldCode]) !== null && _data$column$FieldCod5 !== void 0 && _data$column$FieldCod5.Label ? ((_data$column$FieldCod6 = data[column.FieldCode]) === null || _data$column$FieldCod6 === void 0 ? void 0 : _data$column$FieldCod6.Label) + "%" : 0 + '%' : (_data$column$FieldCod7 = data[column.FieldCode]) === null || _data$column$FieldCod7 === void 0 ? void 0 : _data$column$FieldCod7.Label);
  };

  var headerTemplate = function headerTemplate(column) {

    if (sortField != column.FieldCode) {
      return React__default.createElement("div", {
        style: {
          backgroundColor: getTableFormattingPropertyValue("TableHeaderBackgroundColor")
        },
        onClick: function onClick() {
          return sortTableData(column.FieldCode, column.IsCalculatedColumn);
        },
        className: "header-content"
      }, React__default.createElement("span", {
        style: {
          color: getTableFormattingPropertyValue("TableHeaderFontColor"),
          fontSize: getTableFormattingPropertyValue("TableHeaderFontSize", "px")
        },
        className: "p-column-title"
      }, column.DisplayName));
    }

    return React__default.createElement("div", {
      style: {
        backgroundColor: getTableFormattingPropertyValue("TableHeaderBackgroundColor")
      },
      onClick: function onClick() {
        return sortTableData(column.FieldCode, column.IsCalculatedColumn);
      },
      className: "header-content"
    }, React__default.createElement("span", {
      style: {
        color: getTableFormattingPropertyValue("TableHeaderFontColor"),
        fontSize: getTableFormattingPropertyValue("TableHeaderFontSize", "px")
      },
      className: "p-column-title"
    }, column.DisplayName));
  };

  return isLoading ? React__default.createElement("div", {
    className: 'dl-full-hight dl-flex-content-center spinner-custome'
  }, React__default.createElement(ProgressSpinner, {
    style: {
      width: '40px',
      height: '40px'
    },
    strokeWidth: "4",
    animationDuration: "2s"
  })) : error ? React__default.createElement("div", {
    className: "dl-flex-content-center dl-full-hight",
    style: {
      width: '100%'
    }
  }, React__default.createElement("div", {
    className: "dl-tooltip"
  }, React__default.createElement(SvgRedWarning, {
    className: "red-warning-icon"
  }), React__default.createElement("span", {
    className: "dl-tooltiptext"
  }, error))) : React__default.createElement(DataTable, {
    loading: isTableLoading,
    value: tableData,
    responsiveLayout: "scroll"
  }, tableColumns.map(function (column, index) {
    return React__default.createElement(Column, {
      key: column.FieldCode + "_" + index,
      header: function header() {
        return headerTemplate(column);
      },
      body: function body(data) {
        return cellTemplateBody(data, column);
      }
    });
  }));
};

var PivotTable = forwardRef(function (props, comRef) {
  var _useState = useState(),
      setWidget = _useState[1];

  var _useState2 = useState(true),
      isLoading = _useState2[0],
      setIsLoading = _useState2[1];

  var widgetRefData = useRef(null);
  var tabledata = useRef();
  var pivotGridRef = useRef(null);

  var _useState3 = useState(new PivotGridDataSource({
    store: [],
    fields: []
  })),
      dataSource = _useState3[0],
      setDataSource = _useState3[1];

  useEffect(function () {
    var _props$customChartPro;

    if (props.customChartProps && props.customChartProps.widget) {
      setWidget(props.customChartProps.widget);
      widgetRefData.current = props.customChartProps.widget;
      getPivotTableData();
    }

    (_props$customChartPro = props.customChartProps.dataBinding) === null || _props$customChartPro === void 0 ? void 0 : _props$customChartPro.widgetUpdated.subscribe(function (updatedWidget) {
      var _widgetRefData$curren;

      if ((updatedWidget === null || updatedWidget === void 0 ? void 0 : updatedWidget.key) != ((_widgetRefData$curren = widgetRefData.current) === null || _widgetRefData$curren === void 0 ? void 0 : _widgetRefData$curren.key)) return;
      setWidget(_extends({}, updatedWidget));
    });
    document.addEventListener("resizeStop", handleResize);
    return function () {
      document.removeEventListener("resizeStop", handleResize);
    };
  }, []);
  useEffect(function () {
    if (props.customChartProps.isInEditMode) {
      var _pivotGridRef$current;

      if (pivotGridRef.current && (_pivotGridRef$current = pivotGridRef.current) !== null && _pivotGridRef$current !== void 0 && _pivotGridRef$current.instance && pivotGridRef.current.instance.element && pivotGridRef.current.instance.element().classList) {
        pivotGridRef.current.instance.element().classList.add('force-overflow-hidden');
      }

      setTimeout(function () {
        var _pivotGridRef$current2;

        handleResize();

        if (pivotGridRef.current && (_pivotGridRef$current2 = pivotGridRef.current) !== null && _pivotGridRef$current2 !== void 0 && _pivotGridRef$current2.instance && pivotGridRef.current.instance.element && pivotGridRef.current.instance.element().classList) {
          pivotGridRef.current.instance.element().classList.remove('force-overflow-hidden');
        }
      }, 5);
    }
  }, [props.customChartProps.isInEditMode]);

  var handleResize = function handleResize() {
    var _pivotGridRef$current3, _pivotGridRef$current4;

    if (pivotGridRef.current && (_pivotGridRef$current3 = pivotGridRef.current) !== null && _pivotGridRef$current3 !== void 0 && _pivotGridRef$current3.instance && (_pivotGridRef$current4 = pivotGridRef.current) !== null && _pivotGridRef$current4 !== void 0 && _pivotGridRef$current4.instance.repaint) {
      var _pivotGridRef$current5;

      (_pivotGridRef$current5 = pivotGridRef.current) === null || _pivotGridRef$current5 === void 0 ? void 0 : _pivotGridRef$current5.instance.repaint();
    }
  };

  var getPivotTableData = function getPivotTableData(widget) {
    if (Session.Tenant == 0) {
      tabledata.current = [];
      setIsLoading(false);
      return;
    }

    var dashboardAnalyticsService = new DashboardAnalyticsService();
    dashboardAnalyticsService.getPivotData(props.customChartProps.widget).then(function (res) {
      tabledata.current = res.data;
      setPivotDataSourceFields();
      setIsLoading(false);
    })["catch"](function (err) {
      console.log("Error while Getting Pivot Table Data: ", err);
      tabledata.current = null;
      setPivotDataSourceFields();
      setIsLoading(false);
    });
  };

  var customizeCells = useCallback(function (e) {
    if (e.cell.rowPath && e.cell.rowPath[0] === "Germany" && e.cell.columnPath && e.cell.columnPath[0] === "Export" && e.cell.columnPath[1] === "Air") {
      e.cellElement.children[0].style.padding = "6px";
      e.cellElement.children[0].style.borderRadius = "6px";
      e.cellElement.children[0].style.backgroundColor = "#f4433673";
    }

    if (e.cell.rowPath && e.cell.rowPath[0] === "USA" && e.cell.columnPath && e.cell.columnPath[0] === "Import" && e.cell.columnPath[1] === "Inland") {
      e.cellElement.children[0].style.padding = "6px";
      e.cellElement.children[0].style.borderRadius = "6px";
      e.cellElement.children[0].style.backgroundColor = "#03a9f4ad";
    }
  }, []);
  var onCellClick = useCallback(function (e) {
    if (e.area === 'data') {
      console.log("Pivot Cell Data", e);
    }
  }, []);

  var getColumnNameByColumnId = function getColumnNameByColumnId(fieldName, dataField, datafieldValue) {
    var data = tabledata.current.PivotData.find(function (el) {
      return el[dataField] == datafieldValue;
    });
    return data ? data[fieldName] : '---';
  };

  var setPivotDataSourceFields = function setPivotDataSourceFields() {
    if (tabledata.current && tabledata.current.PivoFields && tabledata.current.PivotMeasures) {
      var pivotRowsAndColumnsFields = tabledata.current.PivoFields.map(function (field) {
        return {
          dataField: field.PivotCode,
          caption: field.DisplayName,
          name: field.PivotName,
          width: 150,
          allowSorting: false,
          allowFiltering: false,
          area: field.PivotCode && field.PivotCode.includes('Row') ? 'row' : 'column',
          expanded: true,
          wordWrapEnabled: true,
          customizeText: function customizeText(cellInfo) {
            return getColumnNameByColumnId(field.PivotName, field.PivotCode, cellInfo.value);
          }
        };
      });
      var pivotDataCellsFields = tabledata.current.PivotMeasures.map(function (measure) {
        return {
          dataField: measure.PivotMeasureCode,
          area: 'data',
          dataType: 'number',
          summaryType: 'sum',
          wordWrapEnabled: true,
          caption: measure.DisplayName
        };
      });
      var AllfieldsData = [].concat(pivotRowsAndColumnsFields, pivotDataCellsFields);
      setDataSource(new PivotGridDataSource({
        store: tabledata.current.PivotData,
        fields: AllfieldsData
      }));
    }
  };

  return React__default.createElement(React__default.Fragment, null, isLoading ? React__default.createElement("div", {
    className: 'dl-full-hight dl-flex-content-center spinner-custome'
  }, React__default.createElement(ProgressSpinner, {
    style: {
      width: '40px',
      height: '40px'
    },
    strokeWidth: "4",
    animationDuration: "2s"
  })) : React__default.createElement(PivotGrid, {
    id: "pivotGridId",
    ref: pivotGridRef,
    dataSource: dataSource,
    allowExpandAll: true,
    showBorders: true,
    className: 'pivot-grid-element',
    onCellClick: onCellClick,
    showRowGrandTotals: false,
    showColumnGrandTotals: false,
    showRowTotals: false,
    showColumnTotals: false,
    wordWrapEnabled: true
  }, React__default.createElement(FieldPanel, {
    showColumnFields: false,
    showDataFields: false,
    showFilterFields: false,
    showRowFields: true,
    texts: {
      rowFieldArea: ""
    },
    allowFieldDragging: false,
    visible: true
  }), React__default.createElement(FieldChooser, {
    enabled: false
  }), React__default.createElement(Scrolling, {
    mode: "virtual"
  })));
});

var CustomChart = function CustomChart(props) {
  var _props$widget2, _props$widget3;

  var project = function project() {
    var _props$widget;

    switch ((_props$widget = props.widget) === null || _props$widget === void 0 ? void 0 : _props$widget.TypeCode) {
      case "kpi":
        return React__default.createElement(KpiChart, {
          customChartProps: props
        });

      case "table":
        return React__default.createElement(TableChart, {
          customChartProps: props
        });

      case "pivot":
        return React__default.createElement(PivotTable, {
          customChartProps: props
        });

      default:
        return React__default.createElement("div", null, "Invalid Type");
    }
  };

  return React__default.createElement(React__default.Fragment, null, React__default.createElement("div", {
    className: "dl-custom-chart-container " + (props.widget && ((_props$widget2 = props.widget) === null || _props$widget2 === void 0 ? void 0 : _props$widget2.TypeCode) == 'table' ? 'dl-table-chart-container' : '') + " " + (props.widget && ((_props$widget3 = props.widget) === null || _props$widget3 === void 0 ? void 0 : _props$widget3.TypeCode) == 'pivot' ? 'dl-pivot-chart-container' : '')
  }, project()));
};

function buildToolTip(widget, value, seriesName, label, index) {
  var padding = widget.TypeCode == 'line' && index > 0 ? 'padding-top: 3px' : '';
  return "\n    <div style=\"min-width: 108px;" + padding + "\">\n        <div style=\"display: flex;align-items: center;margin-bottom: 6px;\">\n            <span style=\"margin: 0; min-height: 10px; min-width: 10px; height: 10px; width: 10px; background-color: transparent; border-radius: 50%; display: inline-block;\"></span>\n            <p style=\" margin: 0; margin-left: 4px; white-space: pre-line; font-family: 'Manrope'; font-style: normal;font-weight: 400; font-size: 11px; line-height: 100%;  color: #323232;\"> " + getTooltiplabel(label, seriesName) + "</p>\n        </div>\n        <hr style=\"margin: 0;border-top: 1px solid #F0F0F0;\">\n        <div style=\"display: flex;align-items: center;margin:0;margin-top: 6px;\">\n            <p style=\"margin: 0;font-family: 'Manrope'; white-space: pre-line; font-style: normal;font-weight: 400; font-size: 11px;line-height: 100%;color: #A4A4A4;\"> " + getMeasureType(seriesName) + "</p>\n            <p style=\"margin: 0;margin-left: 6px; white-space: pre-line; font-family: 'Manrope';font-style: normal;font-weight: 500;font-size: 11px;line-height: 100%;  color: #323232;\">" + getTooltipValue(value, seriesName) + "</p>\n            " + getTooltipPercentage(widget) + " </div>\n    </div>\n    ";
}

function getTooltipPercentage(widget) {
  if (widget.TypeCode != "donut" && widget.TypeCode != "pie") return "";
  return "\n    <p style=\"margin: 0;margin-left: 2px;font-family: 'Manrope';font-style: normal;font-weight: 500;font-size: 11px;line-height: 100%;  color: #A4A4A4;flex: none; order: 1;flex-grow: 0;\">($percentValue)</p>\n    ";
}

function getTooltiplabel(value, seriesMeasure) {
  if (value) return value;
  if (seriesMeasure && seriesMeasure.Name) return '$label' + " - " + seriesMeasure.Name;
  return '$label';
}

var getFormattedValue = function getFormattedValue(value) {
  if (isNaN(value)) return value;

  var _numberSeparatorSettings = getGlobalNumberFormatting();

  var _val = value.toLocaleString('en-US');

  if (_val) {
    var _formattedValue = _val.replaceAll(",", "*");

    _formattedValue = _formattedValue.replaceAll(".", "-");
    _formattedValue = _formattedValue.replaceAll("*", _numberSeparatorSettings.thousandsSeparator);
    _formattedValue = _formattedValue.replaceAll("-", _numberSeparatorSettings.decimalSeparator);
    return _formattedValue;
  }

  return value;
};

function getTooltipValue(value, seriesMeasure) {
  if (!isNullOrUndefined(value)) {
    return seriesMeasure && seriesMeasure.MeasureFieldDataType == 'Percentage' ? getFormattedValue(value) + "%" : getFormattedValue(value);
  }

  return '$dataValue';
}

function getMeasureType(seriesMeasure) {
  if (!seriesMeasure.Name) return "Count:";

  if (seriesMeasure.MeasureCode == "Formula") {
    var formula = JSON.parse(seriesMeasure.Formula);
    return formula.Name;
  }

  return seriesMeasure.Name.replace(/ .*/, '') + ":";
}

var months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
var quarters = ["Q", "Q1", "Q2", "Q3", "Q4"];
function getCategoriesBasedDataSource(chartInfo, seriesMeasures, widget) {
  if (seriesMeasuresIsEmpty(seriesMeasures)) return null;
  handelNullLabels(seriesMeasures);
  return {
    chart: chartInfo,
    categories: buildCategories(seriesMeasures, widget),
    dataset: buildDataSet(seriesMeasures, widget)
  };
}
function getStackedDataSource(chartInfo, seriesMeasures, widget) {
  if (seriesMeasuresIsEmpty(seriesMeasures)) return null;
  handelNullLabels(seriesMeasures);
  return {
    chart: chartInfo,
    categories: buildStackedCategories(seriesMeasures, widget),
    dataset: buildStackedDataSet(seriesMeasures, widget)
  };
}
function getSimpleDataSource(chartInfo, seriesMeasures, widget) {
  if (seriesMeasuresIsEmpty(seriesMeasures)) return null;
  handelNullLabels(seriesMeasures);
  return {
    chart: chartInfo,
    data: buildDataSource(seriesMeasures, widget)
  };
}

function handelNullLabels(seriesMeasures) {
  seriesMeasures.forEach(function (element) {
    element.Values.forEach(function (value) {
      if (!value.Label || value.Label == '') value.Label = 'Not defined';
    });
  });
}

function seriesMeasuresIsEmpty(seriesMeasures) {
  if (!seriesMeasures || seriesMeasures.length == 0) return true;
  var hasValue = false;
  seriesMeasures.forEach(function (item) {
    if (item.Values && item.Values.length > 0) hasValue = true;
  });
  return !hasValue;
}

function buildDataSource(values, widget) {
  var data = [];
  if (!values || !values[0]) return data;
  var series = values[0];
  data = series.Values.map(function (e, index) {
    var item = {
      value: e.Value,
      label: getLabelFormatByDateGroupType(e.Label, widget, false),
      id: {
        GroupById: e.GroupById,
        MeasureFieldId: series.MeasureFieldId,
        MeasureCode: series.MeasureCode,
        Formula: series.Formula
      },
      tooltext: buildToolTip(widget, null, series, e.Label, 0)
    };
    return item;
  });
  return data;
}

function buildDataSet(seriesMeasures, widget) {
  var dataSet = [];
  seriesMeasures.forEach(function (seriesMeasure) {
    var datas = seriesMeasure.Values.map(function (e, index) {
      return {
        value: e.Value,
        id: {
          GroupById: e.GroupById,
          MeasureFieldId: seriesMeasure.MeasureFieldId,
          MeasureCode: seriesMeasure.MeasureCode,
          Formula: seriesMeasure.Formula
        },
        tooltext: buildToolTip(widget, e.Value, seriesMeasure, null, index)
      };
    });
    dataSet.push({
      data: datas,
      seriesname: seriesMeasure.Name,
      anchorBgColor: "inherit",
      anchorBorderColor: "#FFFFFF",
      anchorBorderThickness: "2",
      anchorHoverColor: "inherit",
      parentyaxis: seriesMeasure.PlotasLineChart && seriesMeasure.PlotonSecondAxis ? 'S' : "P",
      renderAs: seriesMeasure.PlotasLineChart ? 'line' : "column"
    });
  });
  return dataSet;
}

function buildStackedDataSet(seriesMeasures, widget) {
  var dataSet = [];
  var series = seriesMeasures[0];
  var uniqueGroups = [];
  series.Values.forEach(function (element) {
    if (!uniqueGroups.find(function (x) {
      return x.code == element.GroupByIdSec;
    })) uniqueGroups.push({
      code: element.GroupByIdSec,
      label: getLabelFormatByDateGroupType(element.LabelSec, widget, true)
    });
  });
  uniqueGroups.forEach(function (group) {
    dataSet.push({
      seriesName: group.label,
      data: buildStackedGroupData(group, series, widget)
    });
  });
  var emptylabels = dataSet.filter(function (e) {
    return e.seriesName == null;
  });
  var filledLabels = dataSet.filter(function (e) {
    return e.seriesName != null;
  });
  dataSet = widget.SecondaryDateGroupCode ? sortDataSet(filledLabels, widget.SecondaryDateGroupCode) : filledLabels;
  return emptylabels.concat(dataSet);
}

function sortDataSet(dataSet, SecondaryDateGroupCode) {
  switch (SecondaryDateGroupCode) {
    case "Year":
      return dataSet.sort(function (s1, s2) {
        return s1.seriesName - s2.seriesName;
      });

    case "Month":
      return dataSet.sort(function (s1, s2) {
        return months.indexOf(s1.seriesName) - months.indexOf(s2.seriesName);
      });

    case "Quarter":
      return dataSet.sort(function (s1, s2) {
        return quarters.indexOf(s1.seriesName) - quarters.indexOf(s2.seriesName);
      });

    case "Month & Year":
      return dataSet.sort(function sort(s1, s2) {
        var aa = s1.seriesName.split(' '),
            bb = s2.seriesName.split(' ');
        return aa[1] - bb[1] || months.indexOf(aa[0]) - months.indexOf(bb[0]);
      });

    case "Quarter & Year":
      return dataSet.sort(function sort(s1, s2) {
        var aa = s1.seriesName.split(' '),
            bb = s2.seriesName.split(' ');
        return aa[1] - bb[1] || quarters.indexOf(aa[0]) - quarters.indexOf(bb[0]);
      });

    default:
      return dataSet;
  }
}

function buildStackedGroupData(secGroup, series, widget) {
  var uniqueGroups = getUniqueGroupOrderdList(series);
  var data = [];
  uniqueGroups.forEach(function (group) {
    var _series$Values$find;

    data.push({
      value: (_series$Values$find = series.Values.find(function (x) {
        return x.GroupById == group.code && x.GroupByIdSec == secGroup.code;
      })) === null || _series$Values$find === void 0 ? void 0 : _series$Values$find.Value,
      id: {
        GroupById: group.code,
        MeasureFieldId: series.MeasureFieldId,
        GroupByIdSec: secGroup.code,
        MeasureCode: series.MeasureCode,
        Formula: series.Formula
      },
      tooltext: buildToolTip(widget, null, series, group.label + " - " + secGroup.label, 0)
    });
  });
  return data;
}

function buildCategories(seriesMeasures, widget) {
  var categories = seriesMeasures[0].Values.map(function (e) {
    return {
      label: getLabelFormatByDateGroupType(e.Label, widget, false)
    };
  });
  var result = [];
  result.push({
    category: categories
  });
  return result;
}

function buildStackedCategories(seriesMeasures, widget) {
  var uniqueGroups = getUniqueGroupOrderdList(seriesMeasures[0]);
  var categories = uniqueGroups.map(function (e) {
    return {
      label: getLabelFormatByDateGroupType(e.label, widget, false)
    };
  });
  var result = [];
  result.push({
    category: categories
  });
  return result;
}

function getUniqueGroupOrderdList(series) {
  var uniqueGroups = [];
  series.Values.forEach(function (element) {
    if (!uniqueGroups.find(function (x) {
      return x.code == element.GroupById;
    })) uniqueGroups.push({
      code: element.GroupById,
      label: element.Label
    });
  });
  return uniqueGroups;
}

var defautFontSize = "12";
var defautFont = "Manrope";
var defautColor = "#323232";
function BuildFusionChartObject(seriesMeasures, widget) {
  switch (widget === null || widget === void 0 ? void 0 : widget.TypeCode) {
    case 'bar':
      if (!widget.SecondaryGroupById) return getBarObject(seriesMeasures, widget);
      return getStackedBarObject(seriesMeasures, widget);

    case 'line':
      if (!widget.SecondaryGroupById) return getLineObject(seriesMeasures, widget);
      return getClusteredLineObject(seriesMeasures, widget);

    case 'pie':
      return getPieObject(seriesMeasures, widget);

    case 'donut':
      return getDonutObject(seriesMeasures, widget);

    case 'column':
      if (!widget.SecondaryGroupById) return getColumnObject(seriesMeasures, widget);
      return getClusteredColumnObject(seriesMeasures, widget);

    default:
      return {};
  }
}

var getWidgetRanges = function getWidgetRanges(widget, value) {
  var gaugeRanges = widget.GaugeRanges && widget.GaugeRanges.length > 0 ? JSON.parse(widget.GaugeRanges) : null;
  if (gaugeRanges == null || isNullOrUndefinedOrEmpty(value)) return [];

  if (value < gaugeRanges[0].minValue) {
    gaugeRanges[0].minValue = value;
  }

  if (value > gaugeRanges[2].maxValue) {
    gaugeRanges[2].maxValue = value;
  }

  return gaugeRanges;
};

var getGaugeObject = function getGaugeObject(data, widget) {
  var fusionObject = getDefaultChartobject();
  fusionObject.type = 'angulargauge';
  var chartInfo = getDefaultChartInfo(widget);
  chartInfo.manageResize = "1";
  chartInfo.valueBelowPivot = "1";
  chartInfo.showValue = "1";
  fusionObject.dataSource = {
    "chart": chartInfo,
    "colorRange": {
      "color": getWidgetRanges(widget, !isNullOrUndefined(data) && !isNullOrUndefined(data === null || data === void 0 ? void 0 : data.Value) ? data === null || data === void 0 ? void 0 : data.Value : 0)
    },
    "dials": {
      "dial": [{
        "value": !isNullOrUndefined(data) && !isNullOrUndefined(data === null || data === void 0 ? void 0 : data.Value) ? data === null || data === void 0 ? void 0 : data.Value : 0
      }]
    }
  };
  return fusionObject;
};

function getBarObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = isGroupByFieldDateTime(widget.GroupByFieldDataType) ? "msbar2d" : "scrollbar2d";
  var chartInfo = getDefaultChartInfo(widget);
  chartInfo.scrollPosition = "right";
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = widget.ShowValueAsNumber ? "0" : "1";
  chartInfo.showLegend = widget.ShowLegend ? '1' : '0';
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "10";
  chartInfo.chartBottomMargin = "10";
  chartInfo.legendPosition = getWidgetLegendPosition(widget);
  chart.dataSource = getCategoriesBasedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}

function getStackedBarObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();

  if (widget.IsStacked) {
    chart.type = isGroupByFieldDateTime(widget.GroupByFieldDataType) ? "stackedbar2d" : "scrollstackedbar2d";
  } else {
    chart.type = isGroupByFieldDateTime(widget.GroupByFieldDataType) ? "msbar2d" : "scrollbar2d";
  }

  var chartInfo = getDefaultChartInfo(widget);
  chartInfo.scrollPosition = "right";
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = widget.ShowValueAsNumber ? "0" : "1";
  chartInfo.showLegend = widget.ShowLegend ? '1' : '0';
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "10";
  chartInfo.chartBottomMargin = "10";
  chartInfo.legendPosition = getWidgetLegendPosition(widget);
  chart.dataSource = getStackedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}

function getClusteredColumnObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();

  if (widget.IsStacked) {
    chart.type = isGroupByFieldDateTime(widget.GroupByFieldDataType) ? "stackedcolumn2d" : "scrollstackedcolumn2d";
  } else {
    chart.type = isGroupByFieldDateTime(widget.GroupByFieldDataType) ? "mscolumn2d" : "scrollcolumn2d";
  }

  var chartInfo = getDefaultChartInfo(widget);
  chartInfo.scrollPosition = "top";
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = widget.ShowValueAsNumber ? "0" : "1";
  chartInfo.showLegend = widget.ShowLegend ? '1' : '0';
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "10";
  chartInfo.chartBottomMargin = "10";
  chartInfo.legendPosition = getWidgetLegendPosition(widget);
  chart.dataSource = getStackedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}

function getClusteredLineObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = isGroupByFieldDateTime(widget.GroupByFieldDataType) ? "msline" : "scrollline2d";
  var chartInfo = getDefaultChartInfo(widget);
  chartInfo.scrollPosition = "top";
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = widget.ShowValueAsNumber ? "0" : "1";
  chartInfo.showLegend = widget.ShowLegend ? '1' : '0';
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "10";
  chartInfo.chartBottomMargin = "10";
  chartInfo.legendPosition = getWidgetLegendPosition(widget);
  chart.dataSource = getStackedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}

function getLineObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = isGroupByFieldDateTime(widget.GroupByFieldDataType) ? "msline" : "scrollline2d";
  var chartInfo = getDefaultChartInfo(widget);
  chartInfo.scrollPosition = "top";
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = widget.ShowValueAsNumber ? "0" : "1";
  chartInfo.showLegend = widget.ShowLegend ? '1' : '0';
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "20";
  chartInfo.chartBottomMargin = "10";
  chartInfo.chartLeftMargin = "20";
  chartInfo.chartRightMargin = "50";
  chartInfo.anchorradius = "5";
  chartInfo.canvasBottomPadding = "10";
  chartInfo.canvasTopPadding = "10";
  chartInfo.legendPosition = getWidgetLegendPosition(widget);
  chart.dataSource = getCategoriesBasedDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}

function getDonutObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = "doughnut2d";
  var chartInfo = getDefaultChartInfo(widget);
  chartInfo.labelFontSize = defautFontSize;
  chartInfo.labelFont = defautFont;
  chartInfo.manageLabelOverflow = "1";
  chartInfo.useEllipsesWhenOverflow = "1";
  chartInfo.enableSmartLabels = "1";
  chartInfo.legendPosition = getWidgetLegendPosition(widget);
  setPieDonutLegends(chartInfo, widget);
  chart.dataSource = getSimpleDataSource(chartInfo, seriesMeasures, widget);
  return chart;
}

function getPieObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = "pie2d";
  var chartInfo = getDefaultChartInfo(widget);
  chartInfo.labelFontSize = defautFontSize;
  chartInfo.labelFont = defautFont;
  chartInfo.manageLabelOverflow = "1";
  chartInfo.useEllipsesWhenOverflow = "1";
  chartInfo.enableSmartLabels = "1";
  setPieDonutLegends(chartInfo, widget);
  chart.dataSource = getSimpleDataSource(chartInfo, seriesMeasures, widget);
  chartInfo.legendPosition = getWidgetLegendPosition(widget);
  return chart;
}

var isGroupByFieldDateTime = function isGroupByFieldDateTime(GroupByFieldDataType) {
  return GroupByFieldDataType == "DateTime" || GroupByFieldDataType == "Date";
};

var isFirstMeasureTypePercentage = function isFirstMeasureTypePercentage(widget) {
  return widget && widget.WidgetMeasures && widget.WidgetMeasures[0] && widget.WidgetMeasures[0].MeasureFieldDataType && widget.WidgetMeasures[0].MeasureFieldDataType == 'Percentage';
};

var isSecondMeasureTypePercentage = function isSecondMeasureTypePercentage(widget) {
  return widget && widget.WidgetMeasures && widget.WidgetMeasures[1] && widget.WidgetMeasures[1].MeasureFieldDataType && widget.WidgetMeasures[1].MeasureFieldDataType == 'Percentage';
};

function getColumnObject(seriesMeasures, widget) {
  var chart = getDefaultChartobject();
  chart.type = isGroupByFieldDateTime(widget.GroupByFieldDataType) ? "mscombidy2d" : "scrollcombidy2d";
  var chartInfo = getDefaultChartInfo(widget);
  chartInfo.scrollPadding = "8";
  chartInfo.flatScrollBars = "1";
  chartInfo.showPercentValues = widget.ShowValueAsNumber ? "0" : "1";
  chartInfo.showLegend = widget.ShowLegend ? '1' : '0';
  chartInfo.scrollPosition = "top";
  setLineDash(chartInfo);
  chartInfo.chartTopMargin = "20";
  chartInfo.chartBottomMargin = "10";
  chartInfo.chartLeftMargin = "20";
  chartInfo.chartRightMargin = "50";
  chartInfo.canvasBottomPadding = "10";
  setDualyAxis(chartInfo, widget, seriesMeasures);
  chart.dataSource = getCategoriesBasedDataSource(chartInfo, seriesMeasures, widget);
  chartInfo.legendPosition = getWidgetLegendPosition(widget);
  return chart;
}

function setLineDash(chartInfo) {
  chartInfo.divLineDashed = "1";
  chartInfo.divLineDashLen = "5";
  chartInfo.divLineDashGap = "5";
  chartInfo.divLineColor = "#E7E7E7";
  chartInfo.divLineAlpha = "100";
  chartInfo.divLineThickness = "1";
}

function getDefaultChartobject() {
  var chart = {};
  chart.type = "column2d";
  chart.width = "100%";
  chart.height = "100%";
  chart.dataFormat = "json";
  return chart;
}

function getDefaultChartInfo(widget) {
  var chart = {};
  chart.theme = "fusion";
  chart.baseFont = defautFont;
  chart.baseFontSize = defautFontSize;
  chart.baseFontColor = defautColor;
  chart.legendCaptionFontSize = defautFontSize;
  chart.legendCaptionFontColor = defautColor;
  chart.legendCaptionFont = defautFont;
  chart.labelFontColor = defautColor;
  chart.labelFont = defautFont;
  chart.legendItemFont = defautFont;
  chart.legendItemFontSize = defautFontSize;
  chart.sYAxisNameFont = defautFont;
  chart.sYAxisNameFontSize = defautFontSize;
  chart.sYAxisNameFontColor = defautColor;
  chart.pYAxisNameFont = defautFont;
  chart.pYAxisNameFontSize = defautFontSize;
  chart.pYAxisNameFontColor = defautColor;
  chart.xAxisNameFont = defautFont;
  chart.xAxisNameFontSize = defautFontSize;
  chart.xAxisNameFontColor = defautColor;
  chart.yAxisNameFont = defautFont;
  chart.yAxisNameFontSize = defautFontSize;
  chart.yAxisNameFontColor = defautColor;
  chart.drawCrossLine = "0";
  chart.xAxisName = widget.ShowXAxisTitle && widget.XAxisTitle ? widget.XAxisTitle : "";
  chart.yAxisName = widget.ShowYAxisTitle && widget.YAxisTitle ? widget.YAxisTitle : "";
  chart.labelDisplay = widget.XAxisLabelsDisplay ? widget.XAxisLabelsDisplay : "Auto";
  chart.slantLabel = "1";
  chart.labelFontSize = widget.XAxisLabelsFontSize ? widget.XAxisLabelsFontSize + "" : "10";
  chart.showValues = widget.ShowValue ? "1" : '0';
  chart.valuePosition = widget.ShowValue && widget.ValuePosition ? widget.ValuePosition : 'Outside';
  var isValuePositionInside = widget.ShowValue && widget.ValuePosition && widget.ValuePosition.toLowerCase() == 'inside';

  if (widget.ShowValue && widget.ValuePosition && isColumnOrBarChart(widget)) {
    chart.placeValuesInside = widget.ValuePosition.toLowerCase() == 'inside' ? "1" : "0";
  }

  if (widget.IsStacked && isValuePositionInside && isColumnOrBarChart(widget)) {
    chart.minPlotHeightForValue = "35";
  }

  if (isValuePositionInside && isPieOrDonutChart(widget)) {
    chart.minAngleForValue = "27";
  }

  chart.numberSuffix = isFirstMeasureTypePercentage(widget) ? "%" : null;

  var _numberSeparatorSettings = getGlobalNumberFormatting();

  chart.thousandSeparator = _numberSeparatorSettings.thousandsSeparator;
  chart.decimalSeparator = _numberSeparatorSettings.decimalSeparator;
  chart.palettecolors = "#369CFB, #FCAF16, #EE5F77, #4AC76F, #935BE0, #FAD337, #32C7C7, #EB67D0, #3C9957, #FF7D4B, #78A6FF, #35838F, #BAA9F3, #C1DA72, #F0AEAD, #246EAB";
  chart.enableSlicing = "0";
  chart.enableRotation = "0";
  chart.chartTopMargin = "10";
  chart.chartBottomMargin = "10";
  chart.chartLeftMargin = "40";
  chart.chartRightMargin = "40";
  chart.toolTipBgColor = "#FFFFFF";
  chart.showToolTipShadow = "1";
  chart.plotHoverEffect = "0";
  chart.scrollHeight = "6";
  chart.scrollWidth = "6";
  chart.plotColorinTooltip = "0";
  chart.formatNumberScale = "0";
  return chart;
}

function setPieDonutLegends(chart, widget) {
  chart.showLabels = widget.LabelsPosition == 'InLegend' ? '0' : '1';
  chart.showLegend = widget.ShowLegend ? '1' : '0';
  setOnChartLegend(chart, widget);
  setChartInLegend(chart, widget);
}

function getWidgetLegendPosition(widget) {
  if (widget.LegendPosition) return widget.LegendPosition;
  return widget.TypeCode == 'pie' || widget.TypeCode == 'donut' ? "right" : "bottom";
}

function isColumnOrBarChart(widget) {
  if (!widget) return false;
  return widget.TypeCode == 'column' || widget.TypeCode == 'bar';
}

function isPieOrDonutChart(widget) {
  if (!widget) return false;
  return widget.TypeCode == 'pie' || widget.TypeCode == 'donut';
}

function setOnChartLegend(chart, widget) {
  if (widget.LabelsPosition != 'InLegend') return;
  chart.legendPosition = getWidgetLegendPosition(widget);
  chart.legendScrollBgColor = "#ffffff";
  chart.legendIconScale = 0.75;
  chart.scrollShowButtons = "0";
  chart.scrollShowButtons = "0";
  chart.legendCaptionAlignment = "right";
  chart.alignLegendWithCanvas = "1";
  chart.showPercentValues = widget.ShowValueAsNumber ? "0" : "1";
  chart.chartRightMargin = "20";
}

function setChartInLegend(chart, widget) {
  if (widget.LabelsPosition == 'InLegend') return;
  chart.pieRadius = "70%";
  chart.showPercentValues = widget.ShowValueAsNumber ? "0" : "1";
}

function setDualyAxis(chartInfo, widget, seriesMeasures) {
  var _seriesMeasures$, _seriesMeasures$2;

  chartInfo.showSecondaryLimits = "0";
  chartInfo.showDivLineSecondaryValue = "0";
  if (!widget) return;
  if (!widget.WidgetMeasures) return;
  if (widget.WidgetMeasures.length < 2) return;
  if (!widget.WidgetMeasures[1].PlotonSecondAxis) return;
  chartInfo.showSecondaryLimits = "1";
  chartInfo.showDivLineSecondaryValue = "1";
  chartInfo.pYAxisNameBorderPadding = 0;
  chartInfo.sYAxisNameBorderPadding = 0;
  chartInfo.sNumberSuffix = isSecondMeasureTypePercentage(widget) ? "%" : null;

  if (isNullOrUndefined(seriesMeasures)) {
    chartInfo.pYAxisName = "";
    chartInfo.sYAxisname = "";
    return;
  }

  var textLength = Math.max(seriesMeasures[0].Name.length, seriesMeasures[1].Name.length);
  chartInfo.pYAxisName = getTitle((_seriesMeasures$ = seriesMeasures[0]) === null || _seriesMeasures$ === void 0 ? void 0 : _seriesMeasures$.Name, textLength);
  chartInfo.sYAxisname = getTitle((_seriesMeasures$2 = seriesMeasures[1]) === null || _seriesMeasures$2 === void 0 ? void 0 : _seriesMeasures$2.Name, textLength);
}

function getTitle(text, maxLenght) {
  var textLenght = text.length;
  if (textLenght >= maxLenght) return text.replace(/\s+/g, '&nbsp;&nbsp;');
  var neededSpaces = Math.ceil((maxLenght - textLenght) / 2);
  var space = '&nbsp;';
  text = space.repeat(neededSpaces) + text + space.repeat(neededSpaces);
  return text.replace(/\s+/g, '&nbsp;&nbsp;');
}

var FusionChartHelper = /*#__PURE__*/function () {
  function FusionChartHelper(widget, chart) {
    this.chart = chart;
    this.widget = widget;
  }

  var _proto = FusionChartHelper.prototype;

  _proto.getPieRadius = function getPieRadius() {
    return this.pieRadius;
  };

  _proto.getLegendWidth = function getLegendWidth() {
    return this.legendWidth;
  };

  _proto.trimChartLabels = function trimChartLabels(chartWidth, chartHeight) {
    var _this$chart;

    if (!this.widget || this.widget.TypeCode != "pie" && this.widget.TypeCode != "donut") return;
    if (!this.chart) return;
    var dataSource = (_this$chart = this.chart) === null || _this$chart === void 0 ? void 0 : _this$chart.dataSource;
    if (!dataSource) return;
    var chartInfo = dataSource.chart;
    if (this.widget.LabelsPosition == 'InLegend') this.handleLegendWidth(chartWidth, chartHeight, chartInfo);else this.handlePieRadius(chartHeight, chartInfo);
  };

  _proto.handlePieRadius = function handlePieRadius(chartHeight, chartInfo) {
    var chartMinumunHeight = 236;
    var startPercantage = 70;
    var stepHeight = 60;
    var step = Math.floor((chartHeight - chartMinumunHeight) / stepHeight);
    if (step < 0) step = 0;
    var pieRadius = startPercantage - step * 10;
    if (pieRadius < 60) pieRadius = 60;

    if (window.innerWidth > 1700) {
      this.pieRadius = pieRadius;
      chartInfo.pieRadius = pieRadius + "%";
      return;
    }

    var reducedValue = Math.floor((1700 - window.innerWidth) / 100) + 1;
    if (reducedValue > 5) reducedValue = 5;
    pieRadius = pieRadius - reducedValue * 5;
    this.pieRadius = pieRadius;
    chartInfo.pieRadius = pieRadius + "%";
  };

  _proto.handleLegendWidth = function handleLegendWidth(chartWidth, chartHeight, chartInfo) {
    if (chartWidth < 730) chartInfo.legendWidth = "200";
    if (chartWidth >= 730) chartInfo.legendWidth = "400";
    if (chartWidth >= 1000) chartInfo.legendWidth = "500";
    this.legendWidth = chartInfo.legendWidth;
  };

  return FusionChartHelper;
}();

var plotColorsDictionary = {
  plotColor0: "#369CFB",
  plotColor1: "#FCAF16",
  plotColor2: "#EE5F77",
  plotColor3: "#4AC76F",
  plotColor4: "#935BE0",
  plotColor5: "#FAD337",
  plotColor6: "#32C7C7",
  plotColor7: "#EB67D0",
  plotColor8: "#3C9957",
  plotColor9: "#FF7D4B",
  plotColor10: "#78A6FF",
  plotColor11: "#35838F",
  plotColor12: "#BAA9F3",
  plotColor13: "#C1DA72",
  plotColor14: "#F0AEAD",
  plotColor15: "#246EAB"
};

var getPlotColorByIndexOrder = function getPlotColorByIndexOrder(index) {
  var _index = index;

  if (_index >= 16) {
    _index = _index % 16;
  }

  switch (_index) {
    case 0:
      return plotColorsDictionary.plotColor0;

    case 1:
      return plotColorsDictionary.plotColor1;

    case 2:
      return plotColorsDictionary.plotColor2;

    case 3:
      return plotColorsDictionary.plotColor3;

    case 4:
      return plotColorsDictionary.plotColor4;

    case 5:
      return plotColorsDictionary.plotColor5;

    case 6:
      return plotColorsDictionary.plotColor6;

    case 7:
      return plotColorsDictionary.plotColor7;

    case 8:
      return plotColorsDictionary.plotColor8;

    case 9:
      return plotColorsDictionary.plotColor9;

    case 10:
      return plotColorsDictionary.plotColor10;

    case 11:
      return plotColorsDictionary.plotColor11;

    case 12:
      return plotColorsDictionary.plotColor12;

    case 13:
      return plotColorsDictionary.plotColor13;

    case 14:
      return plotColorsDictionary.plotColor14;

    case 15:
      return plotColorsDictionary.plotColor15;

    default:
      return "#0566c2";
  }
};

ReactFC.fcRoot(FusionCharts, Charts, FusionTheme);
FusionCharts.options['license']({
  key: 'prD4C-8eiA7A3A3C7E6G4B4A3J4C7B2D3D2nyqE1C3fd1npaE4D4tlA-21D7E4F4F1F1E1F4F1A10A8C2C5F5E2F2D1hwqD1B5D1aG4A19A32twbC6D3G4lhJ-7A9C11A5B-13ddA2I3A1B9B3D7A2B4G2H3H1F-7smC8B2XB4cetB8A7A5mxD3SG4F2tvgB2A3B2E4C3I3C7B3A4A3A2D3D2G4J-7==',
  creditLabel: false
});
var FusionChart = forwardRef(function (props, comRef) {
  var _useState = useState(false),
      dataLoaded = _useState[0],
      setDataLoaded = _useState[1];

  var _useState2 = useState(),
      error = _useState2[0],
      setError = _useState2[1];

  var _useState3 = useState(v4()),
      fusionKey = _useState3[0];

  var chartData = useRef(null);

  var _useState4 = useState(null),
      resizeTimer = _useState4[0],
      setResizeTimer = _useState4[1];

  var axiosController = useRef(new AbortController());
  var widget = useRef();

  var _useState5 = useState({}),
      args = _useState5[0],
      setArgs = _useState5[1];

  var _useState6 = useState(),
      fChart = _useState6[0],
      setFchart = _useState6[1];

  var _useState7 = useState(v4()),
      chartContainerId = _useState7[0];

  var progressRef = useRef(null);
  var fusionRef = useRef(null);
  var fusionChartHelper = useRef();
  var isDummyChangesActive = useRef(false);
  useEffect(function () {
    var _props$dataBinding;

    axiosController.current = new AbortController();
    widget.current = props.widget;
    RefreshData();
    (_props$dataBinding = props.dataBinding) === null || _props$dataBinding === void 0 ? void 0 : _props$dataBinding.widgetUpdated.subscribe(function (updatedWidget) {
      var _widget$current;

      if ((updatedWidget === null || updatedWidget === void 0 ? void 0 : updatedWidget.key) != ((_widget$current = widget.current) === null || _widget$current === void 0 ? void 0 : _widget$current.key)) return;
      widget.current = _extends({}, updatedWidget);

      if (chartData && chartData.current) {
        var _data = JSON.parse(JSON.stringify(chartData.current));

        isDummyChangesActive.current = true;
        handleData(_data, true);
      }
    });
    window.addEventListener("resize", handleResize);
    return function () {
      var _axiosController$curr;

      window.removeEventListener("resize", handleResize);
      setDataLoaded(false);

      if (axiosController && axiosController !== null && axiosController !== void 0 && (_axiosController$curr = axiosController.current) !== null && _axiosController$curr !== void 0 && _axiosController$curr.abort) {
        var _axiosController$curr2;

        axiosController === null || axiosController === void 0 ? void 0 : (_axiosController$curr2 = axiosController.current) === null || _axiosController$curr2 === void 0 ? void 0 : _axiosController$curr2.abort();
      }
    };
  }, []);

  var handleResize = function handleResize() {
    clearTimeout(resizeTimer);
    setResizeTimer(setTimeout(function () {
      HandleSizeChange();
    }, 200));
  };

  useImperativeHandle(comRef, function () {
    return {
      onResizeFinish: function onResizeFinish() {
        HandleSizeChange();
      }
    };
  });

  var HandleSizeChange = function HandleSizeChange() {
    var _widget$current2, _widget$current3;

    if (!widget || ((_widget$current2 = widget.current) === null || _widget$current2 === void 0 ? void 0 : _widget$current2.TypeCode) != "pie" && ((_widget$current3 = widget.current) === null || _widget$current3 === void 0 ? void 0 : _widget$current3.TypeCode) != "donut") return;
    setTimeout(function () {
      var _fusionChartHelper$cu, _fusionChartHelper$cu2, _fusionChartHelper$cu3, _fusionRef$current, _fusionRef$current2, _widget$current4, _fusionChartHelper$cu4, _widget$current5, _fusionChartHelper$cu5;

      var oldPieRadius = fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu = fusionChartHelper.current) === null || _fusionChartHelper$cu === void 0 ? void 0 : _fusionChartHelper$cu.getPieRadius();
      var oldLegendWidth = fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu2 = fusionChartHelper.current) === null || _fusionChartHelper$cu2 === void 0 ? void 0 : _fusionChartHelper$cu2.getLegendWidth();
      fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu3 = fusionChartHelper.current) === null || _fusionChartHelper$cu3 === void 0 ? void 0 : _fusionChartHelper$cu3.trimChartLabels(fusionRef === null || fusionRef === void 0 ? void 0 : (_fusionRef$current = fusionRef.current) === null || _fusionRef$current === void 0 ? void 0 : _fusionRef$current.offsetWidth, fusionRef === null || fusionRef === void 0 ? void 0 : (_fusionRef$current2 = fusionRef.current) === null || _fusionRef$current2 === void 0 ? void 0 : _fusionRef$current2.offsetHeight);
      if (((_widget$current4 = widget.current) === null || _widget$current4 === void 0 ? void 0 : _widget$current4.LabelsPosition) != 'InLegend' && oldPieRadius == (fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu4 = fusionChartHelper.current) === null || _fusionChartHelper$cu4 === void 0 ? void 0 : _fusionChartHelper$cu4.getPieRadius())) return;
      if (((_widget$current5 = widget.current) === null || _widget$current5 === void 0 ? void 0 : _widget$current5.LabelsPosition) == 'InLegend' && oldLegendWidth == (fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : (_fusionChartHelper$cu5 = fusionChartHelper.current) === null || _fusionChartHelper$cu5 === void 0 ? void 0 : _fusionChartHelper$cu5.getLegendWidth())) return;
      setDataLoaded(false);
      setTimeout(function () {
        setDataLoaded(true);
      }, 100);
    }, 100);
  };

  var isTargetWidget = function isTargetWidget(postWidget) {
    return postWidget && (postWidget.TypeCode == 'column' || postWidget.TypeCode == 'bar' || postWidget.TypeCode == 'line');
  };

  var RefreshData = function RefreshData() {
    if (Session.Tenant == 0) {
      handleData(null);
      return;
    }

    var dashboardAnalyticsService = new DashboardAnalyticsService();

    var postWidget = _extends({}, widget.current);

    postWidget.onChange = undefined;
    var signal = undefined;

    if (axiosController) {
      var _axiosController$curr3;

      axiosController === null || axiosController === void 0 ? void 0 : (_axiosController$curr3 = axiosController.current) === null || _axiosController$curr3 === void 0 ? void 0 : _axiosController$curr3.abort();
      var updatedAxiosController = new AbortController();
      axiosController.current = updatedAxiosController;
      signal = updatedAxiosController === null || updatedAxiosController === void 0 ? void 0 : updatedAxiosController.signal;
    }

    var isTargetUsed = isTargetWidget(postWidget) && postWidget.WidgetMeasures && postWidget.WidgetMeasures[1] && postWidget.WidgetMeasures[1].MeasureCode == 'Target';

    if (isTargetUsed) {
      dashboardAnalyticsService.getTargetAnalyticsData(postWidget, signal).then(function (result) {
        handleData(result === null || result === void 0 ? void 0 : result.data);
        chartData.current = result === null || result === void 0 ? void 0 : result.data;
        setDataLoaded(true);
      }, function (error) {
        var _error$response, _error$response$data, _error$response2, _error$response2$data;

        console.log('error', error);
        if (error !== null && error !== void 0 && (_error$response = error.response) !== null && _error$response !== void 0 && (_error$response$data = _error$response.data) !== null && _error$response$data !== void 0 && _error$response$data.ErrorMessage) setError(error === null || error === void 0 ? void 0 : (_error$response2 = error.response) === null || _error$response2 === void 0 ? void 0 : (_error$response2$data = _error$response2.data) === null || _error$response2$data === void 0 ? void 0 : _error$response2$data.ErrorMessage);else setError(error.message);
        setDataLoaded(true);
      });
    } else {
      dashboardAnalyticsService.getData(postWidget, signal).then(function (result) {
        handleData(result === null || result === void 0 ? void 0 : result.data);
        chartData.current = result === null || result === void 0 ? void 0 : result.data;
        setDataLoaded(true);
      }, function (error) {
        var _error$response3, _error$response3$data, _error$response4, _error$response4$data;

        console.log('error', error);
        if (error !== null && error !== void 0 && (_error$response3 = error.response) !== null && _error$response3 !== void 0 && (_error$response3$data = _error$response3.data) !== null && _error$response3$data !== void 0 && _error$response3$data.ErrorMessage) setError(error === null || error === void 0 ? void 0 : (_error$response4 = error.response) === null || _error$response4 === void 0 ? void 0 : (_error$response4$data = _error$response4.data) === null || _error$response4$data === void 0 ? void 0 : _error$response4$data.ErrorMessage);else setError(error.message);
        setDataLoaded(true);
      });
    }
  };

  var handleData = function handleData(data, isDummyChange) {
    var _progressRef$current, _progressRef$current2;

    if (isDummyChange === void 0) {
      isDummyChange = false;
    }

    var chart = BuildFusionChartObject(data, widget.current);
    fusionChartHelper.current = new FusionChartHelper(widget.current, chart);
    fusionChartHelper === null || fusionChartHelper === void 0 ? void 0 : fusionChartHelper.current.trimChartLabels(progressRef === null || progressRef === void 0 ? void 0 : (_progressRef$current = progressRef.current) === null || _progressRef$current === void 0 ? void 0 : _progressRef$current.offsetWidth, progressRef === null || progressRef === void 0 ? void 0 : (_progressRef$current2 = progressRef.current) === null || _progressRef$current2 === void 0 ? void 0 : _progressRef$current2.offsetHeight);
    setArgs(chart);

    if (!isDummyChange) {
      setDataLoaded(true);
    } else {
      setTimeout(function () {
        isDummyChangesActive.current = false;
      }, 100);
    }
  };

  var dataplotClick = function dataplotClick(eventObj, dataObj) {
    props.onSelectDataPoint({
      GroupById: dataObj.id.groupbyid,
      MeasureFieldId: dataObj.id.measurefieldid,
      MeasureCode: dataObj.id.measurecode,
      Formula: dataObj.id.formula,
      Widget: widget.current,
      GroupByIdSec: dataObj.id.groupbyidsec
    });
  };

  var _useResizeObserver = useResizeObserver({
    onResize: function onResize(_ref) {
      var width = _ref.width,
          height = _ref.height;
      fChart === null || fChart === void 0 ? void 0 : fChart.resizeTo(width, height);
    }
  }),
      ref = _useResizeObserver.ref;

  var renderComplete = function renderComplete(chart) {
    setFchart(chart);
  };

  var handlePieDonutChartTooltip = function handlePieDonutChartTooltip(chartDataSource) {
    if (chartDataSource && chartDataSource.data && chartDataSource.data.length) {
      chartDataSource.data.forEach(function (item, index) {
        var color = getPlotColorByIndexOrder(index);
        item.tooltext = item.tooltext.replaceAll("transparent", color);
      });
      setArgs(_extends({}, args, {
        dataSource: chartDataSource
      }));
    }
  };

  var handleOtherChartsTooltip = function handleOtherChartsTooltip(chartDataSource) {
    if (chartDataSource && chartDataSource.dataset && chartDataSource.dataset.length) {
      chartDataSource.dataset.forEach(function (el, index) {
        el.data.forEach(function (item) {
          var color = getPlotColorByIndexOrder(index);
          item.tooltext = item.tooltext.replaceAll("transparent", color);
        });
      });
      setArgs(_extends({}, args, {
        dataSource: chartDataSource
      }));
    }
  };

  var loaded = function loaded(event) {
    var _widget$current6, _widget$current7;

    if (isDummyChangesActive && isDummyChangesActive.current) return;
    var chartDataSource = event.sender.args && event.sender.args.dataSource ? event.sender.args.dataSource : null;
    var isPieOrDonutChart = ((_widget$current6 = widget.current) === null || _widget$current6 === void 0 ? void 0 : _widget$current6.TypeCode) == 'pie' || ((_widget$current7 = widget.current) === null || _widget$current7 === void 0 ? void 0 : _widget$current7.TypeCode) == 'donut';

    if (isPieOrDonutChart) {
      handlePieDonutChartTooltip(chartDataSource);
    } else {
      handleOtherChartsTooltip(chartDataSource);
    }
  };

  return !dataLoaded ? React__default.createElement("div", {
    ref: progressRef,
    className: 'dl-full-hight dl-flex-content-center spinner-custome'
  }, React__default.createElement(ProgressSpinner, {
    style: {
      width: '40px',
      height: '40px'
    },
    strokeWidth: "4",
    animationDuration: "2s"
  })) : !error ? React__default.createElement("div", {
    className: 'db-fc-parent-container',
    ref: fusionRef
  }, React__default.createElement("div", {
    className: 'db-fc-container',
    key: fusionKey,
    ref: ref,
    id: chartContainerId
  }, React__default.createElement(ReactFC, Object.assign({}, args, {
    "fcEvent-dataplotClick": dataplotClick,
    "fcEvent-loaded": loaded,
    onRender: renderComplete
  })))) : React__default.createElement("div", {
    className: "dl-flex-content-center dl-full-hight",
    style: {
      width: '100%'
    }
  }, React__default.createElement("div", {
    className: "dl-tooltip"
  }, React__default.createElement(SvgRedWarning, {
    className: "red-warning-icon"
  }), React__default.createElement("span", {
    className: "dl-tooltiptext"
  }, error)));
});

var _path$1;

function _extends$2() {
  _extends$2 = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends$2.apply(this, arguments);
}

function SvgDeleteIcon(props) {
  return /*#__PURE__*/createElement("svg", _extends$2({
    viewBox: "0 0 20 20",
    fill: "none",
    xmlns: "http://www.w3.org/2000/svg"
  }, props), _path$1 || (_path$1 = /*#__PURE__*/createElement("path", {
    d: "M9 3.512h3v1.264h-1.2v8.217a.65.65 0 01-.176.447.585.585 0 01-.424.185H1.8a.585.585 0 01-.424-.185.65.65 0 01-.176-.447V4.776H0V3.512h3V1.615a.65.65 0 01.176-.447A.585.585 0 013.6.983h4.8c.159 0 .312.067.424.185A.65.65 0 019 1.615v1.897zm.6 1.264H2.4v7.585h7.2V4.776zM4.2 6.672h1.2v3.792H4.2V6.672zm2.4 0h1.2v3.792H6.6V6.672zM4.2 2.247v1.265h3.6V2.247H4.2z",
    fill: "#5FAFDA"
  })));
}

var _path$2;

function _extends$3() {
  _extends$3 = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends$3.apply(this, arguments);
}

function SvgEditIcon(props) {
  return /*#__PURE__*/createElement("svg", _extends$3({
    viewBox: "0 0 20 20",
    fill: "none",
    xmlns: "http://www.w3.org/2000/svg"
  }, props), _path$2 || (_path$2 = /*#__PURE__*/createElement("path", {
    d: "M8.485 5.383l-.943-.943-6.209 6.21v.942h.943l6.21-6.209zm.943-.943l.943-.942-.943-.943-.943.943.943.942zm-6.6 8.486H0v-2.829l8.957-8.956c.26-.26.682-.26.942 0l1.886 1.885c.26.26.26.683 0 .943l-8.957 8.957z",
    fill: "#5FAFDA"
  })));
}

var _path$3;

function _extends$4() {
  _extends$4 = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };
  return _extends$4.apply(this, arguments);
}

function SvgCopyIcon(props) {
  return /*#__PURE__*/createElement("svg", _extends$4({
    viewBox: "0 0 22 22",
    fill: "none",
    xmlns: "http://www.w3.org/2000/svg"
  }, props), _path$3 || (_path$3 = /*#__PURE__*/createElement("path", {
    d: "M3.25 3.5V1.25A.75.75 0 014 .5h9a.75.75 0 01.75.75v10.5a.75.75 0 01-.75.75h-2.25v2.25c0 .414-.338.75-.755.75h-8.99a.751.751 0 01-.755-.75l.002-10.5c0-.414.337-.75.755-.75H3.25zM1.752 5l-.002 9h7.5V5H1.752zM4.75 3.5h6V11h1.5V2h-7.5v1.5z",
    fill: "#5FAFDA"
  })));
}

ReactFC.fcRoot(FusionCharts, Charts$1, FusionTheme);
FusionCharts.options['license']({
  key: 'prD4C-8eiA7A3A3C7E6G4B4A3J4C7B2D3D2nyqE1C3fd1npaE4D4tlA-21D7E4F4F1F1E1F4F1A10A8C2C5F5E2F2D1hwqD1B5D1aG4A19A32twbC6D3G4lhJ-7A9C11A5B-13ddA2I3A1B9B3D7A2B4G2H3H1F-7smC8B2XB4cetB8A7A5mxD3SG4F2tvgB2A3B2E4C3I3C7B3A4A3A2D3D2G4J-7==',
  creditLabel: false
});
var FusionChart$1 = forwardRef(function (props, comRef) {
  var _useState = useState(false),
      dataLoaded = _useState[0],
      setDataLoaded = _useState[1];

  var _useState2 = useState(),
      error = _useState2[0],
      setError = _useState2[1];

  var _useState3 = useState(v4()),
      fusionKey = _useState3[0],
      setFusionKey = _useState3[1];

  var chartData = useRef(null);

  var _useState4 = useState(null),
      resizeTimer = _useState4[0],
      setResizeTimer = _useState4[1];

  var axiosController = useRef(new AbortController());
  var widget = useRef();

  var _useState5 = useState({}),
      args = _useState5[0],
      setArgs = _useState5[1];

  var _useState6 = useState(v4()),
      chartContainerId = _useState6[0];

  var progressRef = useRef(null);
  var fusionRef = useRef(null);
  var isDummyChangesActive = useRef(false);
  useEffect(function () {
    var _props$dataBinding;

    axiosController.current = new AbortController();
    widget.current = props.widget;
    getChartData();
    (_props$dataBinding = props.dataBinding) === null || _props$dataBinding === void 0 ? void 0 : _props$dataBinding.widgetUpdated.subscribe(function (updatedWidget) {
      var _widget$current;

      if ((updatedWidget === null || updatedWidget === void 0 ? void 0 : updatedWidget.key) != ((_widget$current = widget.current) === null || _widget$current === void 0 ? void 0 : _widget$current.key)) return;
      widget.current = _extends({}, updatedWidget);

      if (chartData && chartData.current) {
        var _data = JSON.parse(JSON.stringify(chartData.current));

        isDummyChangesActive.current = true;
        handleData(_data, true);
      }
    });
    window.addEventListener("resize", handleResize);
    return function () {
      var _axiosController$curr;

      window.removeEventListener("resize", handleResize);
      setDataLoaded(false);

      if (axiosController && axiosController !== null && axiosController !== void 0 && (_axiosController$curr = axiosController.current) !== null && _axiosController$curr !== void 0 && _axiosController$curr.abort) {
        var _axiosController$curr2;

        axiosController === null || axiosController === void 0 ? void 0 : (_axiosController$curr2 = axiosController.current) === null || _axiosController$curr2 === void 0 ? void 0 : _axiosController$curr2.abort();
      }
    };
  }, []);

  var handleResize = function handleResize() {
    clearTimeout(resizeTimer);
    setResizeTimer(setTimeout(function () {
      setFusionKey(v4());
    }, 200));
  };

  useImperativeHandle(comRef, function () {
    return {
      onResizeFinish: function onResizeFinish() {
        setFusionKey(v4());
      }
    };
  });

  var getChartData = function getChartData() {
    if (Session.Tenant == 0) {
      handleData(null);
      return;
    }

    var dashboardAnalyticsService = new DashboardAnalyticsService();

    var postWidget = _extends({}, widget.current);

    postWidget.onChange = undefined;
    var signal = undefined;

    if (axiosController) {
      var _axiosController$curr3;

      axiosController === null || axiosController === void 0 ? void 0 : (_axiosController$curr3 = axiosController.current) === null || _axiosController$curr3 === void 0 ? void 0 : _axiosController$curr3.abort();
      var updatedAxiosController = new AbortController();
      axiosController.current = updatedAxiosController;
      signal = updatedAxiosController === null || updatedAxiosController === void 0 ? void 0 : updatedAxiosController.signal;
    }

    dashboardAnalyticsService.getGaugeData(postWidget, signal).then(function (result) {
      handleData(result === null || result === void 0 ? void 0 : result.data);
      chartData.current = result === null || result === void 0 ? void 0 : result.data;
      setDataLoaded(true);
    }, function (error) {
      var _error$response, _error$response$data, _error$response2, _error$response2$data;

      console.log('error', error);
      if (error !== null && error !== void 0 && (_error$response = error.response) !== null && _error$response !== void 0 && (_error$response$data = _error$response.data) !== null && _error$response$data !== void 0 && _error$response$data.ErrorMessage) setError(error === null || error === void 0 ? void 0 : (_error$response2 = error.response) === null || _error$response2 === void 0 ? void 0 : (_error$response2$data = _error$response2.data) === null || _error$response2$data === void 0 ? void 0 : _error$response2$data.ErrorMessage);else setError(error.message);
      setDataLoaded(true);
    });
  };

  var handleData = function handleData(data, isDummyChange) {
    if (isDummyChange === void 0) {
      isDummyChange = false;
    }

    var chart = getGaugeObject(data, widget.current);
    setArgs(chart);

    if (!isDummyChange) {
      setDataLoaded(true);
    } else {
      setTimeout(function () {
        isDummyChangesActive.current = false;
      }, 100);
    }
  };

  return !dataLoaded ? React__default.createElement("div", {
    ref: progressRef,
    className: 'dl-full-hight dl-flex-content-center spinner-custome'
  }, React__default.createElement(ProgressSpinner, {
    style: {
      width: '40px',
      height: '40px'
    },
    strokeWidth: "4",
    animationDuration: "2s"
  })) : !error ? React__default.createElement("div", {
    className: 'db-fc-parent-container',
    ref: fusionRef
  }, React__default.createElement("div", {
    className: 'db-fc-container',
    key: fusionKey,
    id: chartContainerId
  }, React__default.createElement(ReactFC, Object.assign({}, args)))) : React__default.createElement("div", {
    className: "dl-flex-content-center dl-full-hight",
    style: {
      width: '100%'
    }
  }, React__default.createElement("div", {
    className: "dl-tooltip"
  }, React__default.createElement(SvgRedWarning, {
    className: "red-warning-icon"
  }), React__default.createElement("span", {
    className: "dl-tooltiptext"
  }, error)));
});

var WidgetCard = forwardRef(function (props, comRef) {
  var _widget$current13, _widget$current14, _widget$current15;

  var widget = useRef();

  var _useState = useState(false),
      isLoading = _useState[0],
      setIsLoading = _useState[1];

  var fusionChartRef = useRef(null);

  var _useState2 = useState(v4()),
      triggerId = _useState2[0],
      setTriggerId = _useState2[1];

  var editWidget = function editWidget() {
    props.editBtnClicked(widget.current);
  };

  var copyWidget = function copyWidget() {
    if (props && props.copyBtnClicked) {
      var _widget$current;

      props.copyBtnClicked((_widget$current = widget.current) === null || _widget$current === void 0 ? void 0 : _widget$current.key);
    }
  };

  useEffect(function () {
    var _props$dataBinding, _props$dataBinding2;

    widget.current = props.widget;
    (_props$dataBinding = props.dataBinding) === null || _props$dataBinding === void 0 ? void 0 : _props$dataBinding.onEditWidget.subscribe(function (e) {
      var _widget$current2;

      if ((e === null || e === void 0 ? void 0 : e.key) != ((_widget$current2 = widget.current) === null || _widget$current2 === void 0 ? void 0 : _widget$current2.key)) return;
      setIsLoading(true);
      widget.current = _extends({}, e);
      setIsLoading(false);
    });
    (_props$dataBinding2 = props.dataBinding) === null || _props$dataBinding2 === void 0 ? void 0 : _props$dataBinding2.widgetUpdated.subscribe(function (updatedWidget) {
      var _widget$current3;

      if ((updatedWidget === null || updatedWidget === void 0 ? void 0 : updatedWidget.key) != ((_widget$current3 = widget.current) === null || _widget$current3 === void 0 ? void 0 : _widget$current3.key)) return;
      widget.current = _extends({}, updatedWidget);
      setTriggerId(v4());
    });
  }, []);
  useImperativeHandle(comRef, function () {
    return {
      onResizeFinish: function onResizeFinish() {
        var _fusionChartRef$curre;

        fusionChartRef === null || fusionChartRef === void 0 ? void 0 : (_fusionChartRef$curre = fusionChartRef.current) === null || _fusionChartRef$curre === void 0 ? void 0 : _fusionChartRef$curre.onResizeFinish();
      }
    };
  });

  var getCSSPropertyValue = function getCSSPropertyValue(widgetData, propName, propUnit, propDefaultVal) {
    if (propUnit === void 0) {
      propUnit = "";
    }

    if (propDefaultVal === void 0) {
      propDefaultVal = "";
    }

    return widgetData && propName && !isNullOrUndefined(widgetData[propName]) ? widgetData[propName] + propUnit : propDefaultVal || "";
  };

  var GetChartComponent = function GetChartComponent() {
    var _widget$current4, _widget$current5, _widget$current6, _widget$current7, _widget$current8;

    if (isLoading || !((_widget$current4 = widget.current) !== null && _widget$current4 !== void 0 && _widget$current4.TypeCode)) return "";
    if (((_widget$current5 = widget.current) === null || _widget$current5 === void 0 ? void 0 : _widget$current5.TypeCode) == "kpi" || ((_widget$current6 = widget.current) === null || _widget$current6 === void 0 ? void 0 : _widget$current6.TypeCode) == "table" || ((_widget$current7 = widget.current) === null || _widget$current7 === void 0 ? void 0 : _widget$current7.TypeCode) == "pivot") return React__default.createElement(CustomChart, {
      isInEditMode: props.isInEditMode,
      isPreviewModeActive: props.isPreviewModeActive,
      dataBinding: props.dataBinding,
      widget: widget.current,
      widgetRef: props.widgetRef,
      onSelectDataPoint: props.onSelectDataPoint
    });
    if (((_widget$current8 = widget.current) === null || _widget$current8 === void 0 ? void 0 : _widget$current8.TypeCode) == "gauge") return React__default.createElement(FusionChart$1, {
      ref: fusionChartRef,
      dataBinding: props.dataBinding,
      widget: widget.current,
      widgetRef: props.widgetRef,
      onSelectDataPoint: props.onSelectDataPoint
    });
    return React__default.createElement(FusionChart, {
      ref: fusionChartRef,
      dataBinding: props.dataBinding,
      widget: widget.current,
      widgetRef: props.widgetRef,
      onSelectDataPoint: props.onSelectDataPoint
    });
  };

  var isTableWidget = function isTableWidget() {
    return widget && widget.current && widget.current.TypeCode == 'table';
  };

  var isPivotWidget = function isPivotWidget() {
    return widget && widget.current && widget.current.TypeCode == 'pivot';
  };

  var isTableOrPivotWidget = function isTableOrPivotWidget() {
    return widget && widget.current && (widget.current.TypeCode == 'table' || widget.current.TypeCode == 'pivot');
  };

  var getwidgetTitlesContainer = function getwidgetTitlesContainer() {
    return isTableOrPivotWidget() ? "widget-table-titles-container" : "widget-titles-container";
  };

  var drilldownToTableData = function drilldownToTableData() {
    props.onSelectDataPoint({
      GroupById: undefined,
      MeasureFieldId: undefined,
      MeasureCode: undefined,
      Formula: undefined,
      Widget: widget.current,
      GroupByIdSec: undefined
    });
  };

  var isThereTitle = function isThereTitle() {
    var _widget$current9, _widget$current10;

    return ((_widget$current9 = widget.current) === null || _widget$current9 === void 0 ? void 0 : _widget$current9.Title) && ((_widget$current10 = widget.current) === null || _widget$current10 === void 0 ? void 0 : _widget$current10.Title.length) > 0;
  };

  var isThereSubtitle = function isThereSubtitle() {
    var _widget$current11, _widget$current12;

    return ((_widget$current11 = widget.current) === null || _widget$current11 === void 0 ? void 0 : _widget$current11.Subtitle) && ((_widget$current12 = widget.current) === null || _widget$current12 === void 0 ? void 0 : _widget$current12.Subtitle.length) > 0;
  };

  var tableHintMessage = function tableHintMessage() {
    return React__default.createElement("span", {
      className: "table-hint-container " + (isPivotWidget() ? "min-width-105" : "") + " " + (isTitleAligmentRight() ? "padding-left-12" : "")
    }, React__default.createElement("span", {
      style: {
        visibility: isTableWidget() ? "visible" : "hidden"
      }
    }, isTableWidget() ? "Showing the first 100 rows" : ""), !props.isInEditMode && !props.isPreviewModeActive && isTableWidget() && React__default.createElement(React__default.Fragment, null, React__default.createElement("span", null, "("), React__default.createElement("span", {
      onClick: drilldownToTableData,
      className: "show-more-records"
    }, " Show More Records "), React__default.createElement("span", null, ")")));
  };

  var isTitleAligmentRight = function isTitleAligmentRight() {
    return widget && widget.current && widget.current.TitleAlignment && widget.current.TitleAlignment.toLowerCase() == 'right';
  };

  return React__default.createElement("div", {
    className: "dl-panel",
    key: props.el.i
  }, React__default.createElement("header", {
    key: triggerId,
    className: "widget-header " + (isPivotWidget() ? "pivot-header" : "")
  }, React__default.createElement("div", {
    className: "header-contents " + (isTableWidget() ? "border-bottom-light-white" : "")
  }, isTitleAligmentRight() && isTableOrPivotWidget() && tableHintMessage(), React__default.createElement("div", {
    style: {
      cursor: props.isInEditMode ? "move" : "default",
      justifyContent: isTableOrPivotWidget() ? getCSSPropertyValue(widget === null || widget === void 0 ? void 0 : widget.current, 'TitleAlignment', '', undefined) : undefined,
      flexDirection: isTitleAligmentRight() ? "row-reverse" : "row"
    },
    className: getwidgetTitlesContainer()
  }, React__default.createElement("div", {
    className: "title-container",
    style: {
      justifyContent: getCSSPropertyValue(widget === null || widget === void 0 ? void 0 : widget.current, 'TitleAlignment', '', undefined)
    }
  }, React__default.createElement(Popup, {
    className: "title-tooltip-container",
    trigger: React__default.createElement("div", {
      className: "title",
      style: {
        fontSize: getCSSPropertyValue(widget.current, 'TitleFontSize', 'px'),
        color: getCSSPropertyValue(widget.current, 'TitleFontColor', ''),
        textAlign: getCSSPropertyValue(widget === null || widget === void 0 ? void 0 : widget.current, 'TitleAlignment', '', undefined)
      }
    }, (_widget$current13 = widget.current) === null || _widget$current13 === void 0 ? void 0 : _widget$current13.Title),
    position: ['top center'],
    arrowStyle: {
      color: '#262626'
    },
    offsetY: -8,
    on: ['hover', 'focus']
  }, React__default.createElement("span", {
    className: "widget-title-tooltip-text"
  }, (_widget$current14 = widget.current) === null || _widget$current14 === void 0 ? void 0 : _widget$current14.Title))), isTableOrPivotWidget() && isThereTitle() && isThereSubtitle() && React__default.createElement("div", {
    className: "titles-vertical-seeparator"
  }), React__default.createElement("div", {
    className: isThereSubtitle() ? "subtitle" : "subtitle margin-top-4",
    style: {
      fontSize: getCSSPropertyValue(widget.current, 'SubtitleFontSize', 'px'),
      textAlign: getCSSPropertyValue(widget === null || widget === void 0 ? void 0 : widget.current, 'TitleAlignment', '', undefined),
      color: getCSSPropertyValue(widget.current, 'SubtitleFontColor', ''),
      minWidth: isThereSubtitle() && isTableOrPivotWidget() ? '5%' : undefined
    }
  }, (_widget$current15 = widget.current) === null || _widget$current15 === void 0 ? void 0 : _widget$current15.Subtitle)), !isTitleAligmentRight() && isTableOrPivotWidget() && tableHintMessage(), props.isInEditMode && React__default.createElement("div", {
    className: isThereTitle() ? "widget-options-container" : "widget-options-container margin-top-6"
  }, React__default.createElement(SvgEditIcon, {
    onClick: function onClick() {
      return editWidget();
    }
  }), React__default.createElement(SvgCopyIcon, {
    onClick: function onClick() {
      return copyWidget();
    }
  }), React__default.createElement(SvgDeleteIcon, {
    onClick: function onClick() {
      var _props$widget;

      return props.deleteBtnClicked((_props$widget = props.widget) === null || _props$widget === void 0 ? void 0 : _props$widget.key);
    }
  })))), React__default.createElement("div", {
    className: "dl-dashboard-widget"
  }, GetChartComponent()));
});

function deepClone(obj) {
  return JSON.parse(JSON.stringify(obj));
}

var ResponsiveReactGridLayout = WidthProvider(Responsive);

var DashboardDesigner = function DashboardDesigner(props) {
  var array12 = Array.from({
    length: 12
  }).fill(null);
  var minimumPlaceholderRowsCount = 12;

  var _useState = useState({
    lg: []
  }),
      layouts = _useState[0],
      setLayouts = _useState[1];

  var _useState2 = useState(Array.from({
    length: minimumPlaceholderRowsCount
  }).fill(null)),
      placeholderRowsArray = _useState2[0],
      setplaceholderRowsArray = _useState2[1];

  var _useState3 = useState(undefined),
      dateFilter = _useState3[0];

  var _useState4 = useState(false),
      isInEditMode = _useState4[0],
      setisInEditMode = _useState4[1];

  var widgetsDictionaryRef = useRef({});

  var _useState5 = useState({
    lg: []
  }),
      widgets = _useState5[0],
      setWidgets = _useState5[1];

  var gridLayoutComponent = useRef(null);
  var entered = useRef(false);
  var widgetCardRef = useRef([]);
  useEffect(function () {
    FillLayout(props.Widgetlayouts);
  }, [props.Widgetlayouts]);
  useEffect(function () {
    var _props$dataBinding;

    if (props !== null && props !== void 0 && (_props$dataBinding = props.dataBinding) !== null && _props$dataBinding !== void 0 && _props$dataBinding.onDeleteWidgetConfirmed) {
      props.dataBinding.onDeleteWidgetConfirmed.subscribe(function (deleteProps) {
        if (deleteProps) {
          deletePanel(deleteProps);
        }
      });
    }
  }, []);

  var FillLayout = function FillLayout(dashboardLayouts) {
    var clone = deepClone(dashboardLayouts);
    var layout = createLayout(clone);
    FillWidgetDictionary(clone);
    setWidgets(clone);
    setLayouts(layout);
    UpdatePlaceholderDimensions(layout);
  };

  var FillWidgetDictionary = function FillWidgetDictionary(clone) {
    for (var _iterator = _createForOfIteratorHelperLoose(clone.lg), _step; !(_step = _iterator()).done;) {
      var item = _step.value;
      widgetsDictionaryRef.current[item.key] = item;
    }
  };

  var createLayout = function createLayout(dashboardLayouts) {
    var layout = {
      lg: []
    };
    dashboardLayouts.lg.map(function (w) {
      return w.Layout;
    }).forEach(function (e) {
      layout.lg.push(e);
    });
    return layout;
  };

  useEffect(function () {
    setisInEditMode(props.editMode);
  }, [props.editMode]);

  function onLayoutChange(layout, changedLayouts) {
    UpdateWidget(changedLayouts);
    setLayouts(changedLayouts);
    UpdatePlaceholderDimensions(changedLayouts);

    if (entered.current == false) {
      entered.current = true;
      return;
    }

    props.onLayoutChange(changedLayouts);
  }

  function onResizeStop(event, oldItem) {
    var _widgetCardRef$curren, _widgetCardRef$curren2;

    document.dispatchEvent(new CustomEvent("resizeStop", {}));
    widgetCardRef === null || widgetCardRef === void 0 ? void 0 : (_widgetCardRef$curren = widgetCardRef.current) === null || _widgetCardRef$curren === void 0 ? void 0 : (_widgetCardRef$curren2 = _widgetCardRef$curren.find(function (x) {
      return x.i == oldItem.i;
    })) === null || _widgetCardRef$curren2 === void 0 ? void 0 : _widgetCardRef$curren2.ref.onResizeFinish(widgetsDictionaryRef.current[oldItem.i].key);
  }

  function UpdateWidget(changedLayouts) {
    for (var _iterator2 = _createForOfIteratorHelperLoose(changedLayouts.lg), _step2; !(_step2 = _iterator2()).done;) {
      var item = _step2.value;

      if (widgetsDictionaryRef.current[item.i]) {
        widgetsDictionaryRef.current[item.i].Layout = item;
      }
    }
  }

  var confirmDeleteWidget = function confirmDeleteWidget(id) {
    var deleteWidgetProps = {
      id: id,
      layouts: layouts,
      widgets: widgets
    };
    props.onWidgetDelete(deleteWidgetProps);
  };

  function deletePanel(deleteProps) {
    var newLayouts = {
      lg: deleteProps.layouts.lg.filter(function (e) {
        return e.i != deleteProps.id;
      })
    };
    setLayouts(newLayouts);
    var newWidgets = {
      lg: deleteProps.widgets.lg.filter(function (e) {
        return e.key != deleteProps.id;
      })
    };
    FillWidgetDictionary(newWidgets);
    setWidgets(newWidgets);
    props.onLayoutChange(newLayouts);
  }

  function editWidget(widget) {
    widget.ChangeSetOp = 'Update';
    props.openEditWidget(widget);
  }

  function copyWidget(key) {
    if (props && props.onCopyWidget) {
      props.onCopyWidget(key);
    }
  }

  function UpdatePlaceholderDimensions(layouts) {
    if (!layouts) return;
    var max = minimumPlaceholderRowsCount;

    for (var _iterator3 = _createForOfIteratorHelperLoose(layouts['lg']), _step3; !(_step3 = _iterator3()).done;) {
      var layout = _step3.value;
      if (layout.y + layout.h > max) max = layout.y + layout.h;
    }

    var newPlaceholderRows = max + 2;
    newPlaceholderRows = newPlaceholderRows < minimumPlaceholderRowsCount ? minimumPlaceholderRowsCount : newPlaceholderRows;
    setplaceholderRowsArray(Array.from({
      length: newPlaceholderRows
    }).fill(null));
  }

  var _useResizeObserver = useResizeObserver({
    onResize: function onResize(_ref) {
      _objectDestructuringEmpty(_ref);

      window.dispatchEvent(new Event('resize'));
    }
  }),
      ref = _useResizeObserver.ref;

  return React__default.createElement("div", {
    ref: ref,
    className: "dl-full-hight-scroll"
  }, React__default.createElement("div", {
    className: "dl-box"
  }, React__default.createElement("div", {
    className: "dl-row dl-content"
  }, React__default.createElement("div", {
    className: "dl-grid-container"
  }, React__default.createElement("div", {
    className: "dl-grid dl-grid--bordered"
  }, isInEditMode ? placeholderRowsArray.map(function (e, i) {
    return array12.map(function (e, i) {
      return React__default.createElement("div", {
        key: i,
        style: {
          height: layoutGridProps.rowHeight
        }
      });
    });
  }) : null), React__default.createElement(ResponsiveReactGridLayout, Object.assign({
    ref: gridLayoutComponent
  }, layoutGridProps, {
    layouts: layouts,
    compactType: "vertical",
    draggableCancel: ".widget-options-container",
    isDraggable: isInEditMode,
    isResizable: isInEditMode,
    onWidthChange: function onWidthChange() {},
    onResizeStop: onResizeStop,
    onLayoutChange: onLayoutChange
  }), widgets.lg.map(function (widget, i) {
    var _widget$Layout, _widget$Layout2;

    return React__default.createElement("div", {
      key: (_widget$Layout = widget.Layout) === null || _widget$Layout === void 0 ? void 0 : _widget$Layout.i
    }, React__default.createElement(WidgetCard, {
      isInEditMode: isInEditMode,
      key: (_widget$Layout2 = widget.Layout) === null || _widget$Layout2 === void 0 ? void 0 : _widget$Layout2.i,
      widget: widget,
      widgetRef: widgetsDictionaryRef.current[widget.key],
      el: widget,
      dataBinding: props.dataBinding,
      dateFilter: dateFilter,
      editBtnClicked: editWidget,
      deleteBtnClicked: confirmDeleteWidget,
      onSelectDataPoint: props.onSelectDataPoint,
      copyBtnClicked: copyWidget,
      ref: function ref(el) {
        var _widget$Layout3;

        widgetCardRef.current[i] = {
          ref: el,
          i: (_widget$Layout3 = widget.Layout) === null || _widget$Layout3 === void 0 ? void 0 : _widget$Layout3.i
        };
      }
    }));
  }))))));
};

var layoutGridProps = {
  rowHeight: 50,
  className: "layout",
  cols: {
    lg: 12,
    md: 6,
    sm: 2,
    xs: 1,
    xxs: 1
  },
  useCSSTransforms: true
};

var Dashboard = function Dashboard(props) {
  var _previewWidget$Layout, _previewWidget$Layout2;

  var Widgetlayouts = useRef({
    lg: []
  });

  var _useState = useState({
    lg: []
  }),
      widgetlayoutsState = _useState[0],
      setWidgetlayoutsState = _useState[1];

  var _useState2 = useState(false),
      isInEditMode = _useState2[0],
      setIsInEditMode = _useState2[1];

  var _useState3 = useState(props.isPreviewMode !== undefined ? props.isPreviewMode : false),
      isPreviewModeActive = _useState3[0],
      setIsPreviewModeActive = _useState3[1];

  var _useState4 = useState(props.widgetData),
      previewWidget = _useState4[0],
      setPreviewWidget = _useState4[1];

  useEffect(function () {
    Session.CurrentUserId = props.userId;
    Session.Tenant = props.tenant;
    Session.Token = props === null || props === void 0 ? void 0 : props.token;
    Session.NumberFormatCode = props === null || props === void 0 ? void 0 : props.numberFormatCode;
    setIsPreviewModeActive(props.isPreviewMode);
    props.dataBinding.onGetLayouts.subscribe(function (layouts) {
      Widgetlayouts.current = layouts;
      setWidgetlayoutsState(deepClone(Widgetlayouts.current));
    });
    props.dataBinding.isOnEditLayout.subscribe(function (e) {
      setIsInEditMode(e);
    });
    props.dataBinding.onAddWidget.subscribe(function (w) {
      Widgetlayouts.current.lg.push(w);
      setWidgetlayoutsState(deepClone(Widgetlayouts.current));
    });
  }, []);
  useEffect(function () {
    if (props.widgetData) {
      setPreviewWidget(JSON.parse(JSON.stringify(props.widgetData)));
    }
  }, [props.widgetData]);

  var copyWidget = function copyWidget(key) {
    if (props && props.onCopyWidget) {
      props.onCopyWidget(key);
    }
  };

  var updateWidgets = function updateWidgets(layouts) {
    var layoutsDic = {};
    Object.keys(layouts).forEach(function (sizeType) {
      if (layouts[sizeType].length > 0) {
        layouts[sizeType].map(function (layout) {
          layoutsDic[layout.i] = layout;
        });
        return;
      }
    });
    var newWidgetlayouts = [];
    Widgetlayouts.current.lg.forEach(function (widget) {
      if (layoutsDic[widget.key]) {
        widget.StartPotistion = layoutsDic[widget.key].x + "," + layoutsDic[widget.key].y;
        widget.EndPosition = layoutsDic[widget.key].w + "," + layoutsDic[widget.key].h;
        widget.Layout = layoutsDic[widget.key];
        newWidgetlayouts.push(widget);
      }
    });
    Widgetlayouts.current = {
      lg: newWidgetlayouts
    };
    props.onChangeLayouts(Widgetlayouts.current);
  };

  return isPreviewModeActive && previewWidget ? createElement("div", {
    style: {
      width: "420px",
      height: "310px"
    }
  }, createElement("div", {
    style: {
      height: "100%"
    },
    key: "widget-" + (previewWidget === null || previewWidget === void 0 ? void 0 : (_previewWidget$Layout = previewWidget.Layout) === null || _previewWidget$Layout === void 0 ? void 0 : _previewWidget$Layout.i)
  }, createElement(WidgetCard, {
    isInEditMode: isInEditMode,
    isPreviewModeActive: isPreviewModeActive,
    key: previewWidget === null || previewWidget === void 0 ? void 0 : (_previewWidget$Layout2 = previewWidget.Layout) === null || _previewWidget$Layout2 === void 0 ? void 0 : _previewWidget$Layout2.i,
    widget: previewWidget,
    widgetRef: previewWidget,
    el: previewWidget,
    dataBinding: props.dataBinding,
    dateFilter: undefined,
    editBtnClicked: function editBtnClicked() {
      return true;
    },
    deleteBtnClicked: function deleteBtnClicked() {
      return true;
    },
    onSelectDataPoint: function onSelectDataPoint() {
      return true;
    }
  }))) : createElement("div", {
    className: 'dl-full-hight'
  }, createElement("div", {
    className: 'dl-dashboard-body'
  }, createElement(DashboardDesigner, {
    onSelectDataPoint: props.onSelectDataPoint,
    Widgetlayouts: widgetlayoutsState,
    dataBinding: props.dataBinding,
    editMode: isInEditMode,
    onLayoutChange: updateWidgets,
    onWidgetDelete: function onWidgetDelete(deleteProps) {
      return props.openDeleteWidgetConfirmWindow(deleteProps);
    },
    onCopyWidget: copyWidget,
    openEditWidget: function openEditWidget(widget) {
      return props.openEditWidget(widget);
    }
  })));
};

export default Dashboard;
//# sourceMappingURL=index.modern.js.map
